#!/bin/ksh
#@
#@ *********************************************************************
#@
#@ Script Name: h27367a1.ec
#@
#@ Description: This is a mainline script program. This script controls
#@              the addition of SSA disks.
#@
#@              Each mainline script sets must set up the proper
#@              environment:
#@
#@                1) initialize environment variables from the
#@                   primary (required) and override (optiona)
#@                   environment control files
#@                2) Initialize master log name
#@                3) Invoke the script buildDiskSets.stsm to build the
#@                   diskSet<xx> environment variables.
#@
#@ Input:
#@
#@    Command Line Input:
#@
#@    StorageSmart Build Environment Variables: all
#@
#@ Output: variable dependent on the environment control variables
#@
#@ Change Cntl  Name                 Description
#@ -----------  ------------------   ----------------------------------
#@ 11/10/1999   Chris Knapp          Create
#@ 11/14/2000   Howard Jiang         Modify for C01 and 6+P+S
#@
#@ ********************************************************************

# -----------------------------------------------------------------------------
# Development control flags for script execution
# -----------------------------------------------------------------------------

botherMe='no'  # When selections are skipped, do you want to be prompted?

# -------
# Phase 1
# -------

skipCheckSystemType='no'
skipCreateLogDirectory='no'
skipCheckOperatingSystemLevel='no'
skipCleanUpJbodSystemDisks='no'
skipSanityCheckVolumeGroups='no'
skipVerifyCommunicationWithTsmServer='no'
skipVerifyTsmStuff='no'
skipValidateABLoops='no'
skipCreateOrderedLoopLists='no'
skipVerifyBalancedLoops='no'
skipVerifyVdisk='no'

# -------
# Phase 2
# -------

skipVerifySSAInstallation='no'       # Can always be skipped (if pre-req'd)
skipVerifyCabling='no'               # Can always be skipped (if pre-req'd)
skipBuildPdiskListsForLoop='no'
skipCreateSpares='no'
skipCreateArrays='no'
skipExtendADSMvgRaid='no'
skipCreateJFS='no'
skipCreateADSMresource='no'
skipDefineADSMvol='no'
skipSaveSystemInfo='no'

# -----------------------------------------------------------------------------
# Define default inputs
# -----------------------------------------------------------------------------

export scriptName='h27367a1.ec'
export netStoreMachineType='C01'
systemModel='4C'
modelName='H70'
response=''
minimumOSVersion='4'
minimumOSRelease='3'
minimumOSLevel='3'
raid5mesInfoFile="/adsmlog/$scriptName.progressLog"
featureCode1='3207'
featureCode2='3495'
featureCode3='3495'
tsmvg='adsmvg'  # Default AIX TSM volume group
minTsmDbValue='0'  # Minimum TSM database size user is allowed to allocate
maxTsmDbValue='18'  # Maximum TSM database size user is allowed to allocate
subprocessTypeFunction='function'
subprocessTypeScript='script'
mesBuildscrDir='/cd/buildscr'
phase1acompleted='no'
phase1bcompleted='no'
phase2completed='no'

# -----------------------------------------------------------------------------
# Display error messages to STDERR
# -----------------------------------------------------------------------------

writeErr ()
{
  print "$*" >&2
}

# -----------------------------------------------------------------------------
# Displays a message when entering a subprocess (function / script)
# -----------------------------------------------------------------------------

function enteringSubprocess {

  currentTimeAndDate=`date "+%I:%M %p - %m/%d/%Y"`
  if [ "$2" = 'function' ]
  then
    echo "INFO: ($scriptName) > Entering $2: $1{}"
    echo "                      [ $currentTimeAndDate ]"
    echo
  elif [ "$2" = 'script' ]
  then
    echo "INFO: ($scriptName) > Entering $2: $1"
    echo "                      [ $currentTimeAndDate ]"
    echo
  fi

}  # END enteringSubprocess

# -----------------------------------------------------------------------------
# Displays a message when exiting a subprocess (function / script)
# -----------------------------------------------------------------------------

function exitingSubprocess {

  currentTimeAndDate=`date "+%I:%M %p - %m/%d/%Y"`
  if [ "$2" = 'function' ]
  then
    echo "INFO: ($scriptName) < Exiting $2: $1{}"
    echo "                      [ $currentTimeAndDate ]"
    echo
  elif [ "$2" = 'script' ]
  then
    echo "INFO: ($scriptName) < Exiting $2: $1"
    echo "                      [ $currentTimeAndDate ]"
    echo
  fi

}  # END exitingSubprocess

# -----------------------------------------------------------------------------
# Displays a message when skipping a subprocess (function / script)
# -----------------------------------------------------------------------------

function skippingSubprocess {

  currentTimeAndDate=`date "+%I:%M %p - %m/%d/%Y"`
  if [ "$2" = 'function' ]
  then
    echo "INFO: ($scriptName) + Skipping $2: $1{}"
    echo "                      [ $currentTimeAndDate ]"
    echo
  elif [ "$2" = 'script' ]
  then
    echo "INFO: ($scriptName) + Skipping $2: $1"
    echo "                      [ $currentTimeAndDate ]"
    echo
  fi

}  # END skippingSubprocess

# -----------------------------------------------------------------------------
# Prompts the user to type 'c' and Enter to continue
# -----------------------------------------------------------------------------

function cAndEnterToContinue {

  response=''
  until [ "$response" = 'c' ] || [ "$response" = 'C' ]
  do
    read response?"Press 'C' and [Enter] to continue: "
    echo
  done

}  # END cAndEnterToContinue

# -----------------------------------------------------------------------------
# Prompts the user to type 'e' and Enter to exit
# -----------------------------------------------------------------------------

function eAndEnterToExit {

  response=''
  until [ "$response" = 'e' ] || [ "$response" = 'E' ]
  do
    read response?"Press 'E' and [Enter] to exit: "
    echo
  done
  programRC=1
  exit $programRC

}  # END eAndEnterToExit

# -----------------------------------------------------------------------------
# Prompts the user to type 'c' and Enter to continue or 'e' and Enter to exit
# -----------------------------------------------------------------------------

function cOrEAndEnterToContinue {

  response=''
  until [ "$response" = 'c' ] || [ "$response" = 'C' ] || [ "$response" = 'e' ] || [ "$response" = 'E' ]
  do
    read response?"Press 'C' and [Enter] to continue or 'E' and [Enter] to exit: "
    echo
  done

  if [ "$response" = 'e' ] || [ "$response" = 'E' ]
  then
    programRC=1
    exit $programRC
  fi

}  # END cOrEAndEnterToContinue

# -----------------------------------------------------------------------------
# Routine to confirm selection(s)
# -----------------------------------------------------------------------------

function botherMeRoutine {

  if [ "$botherMe" = 'yes' ]
  then
    cAndEnterToContinue
  fi

}  # END botherMeRoutine

# -----------------------------------------------------------------------------
# Tell the user to expand the screen
# -----------------------------------------------------------------------------

function expandScreen {

  subprocessName='expandScreen'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  clear
  echo
  echo "+-----------------------------------------------------------------------------+"
  echo "|                                                                             |"
  echo "| ! ATTENTION !                                                               |"
  echo "|                                                                             |"
  echo "| Information will be displayed on the following screens that will not fit in |"
  echo "| a window which was launched with the default size of 80x24.                 |"
  echo "|                                                                             |"
  echo "| You should increase the size of this window to at least 100x35 in order to  |"
  echo "| effectively view information presented in this script.                      |"
  echo "|                                                                             |"
  echo "| The window size can be increased by:                                        |"
  echo "|                                                                             |"
  echo "|  -- Maximizing to full screen with the maximize button                      |"
  echo "|     (upper right hand corner)                                               |"
  echo "|                                                                             |"
  echo "|  -- Stretching the window height.  Position the mouse on the top or bottom  |"
  echo "|     edge of the screen.  Notice that the pointer changes to indicate the    |"
  echo "|     mouse is positioned to stretch the screen.  Hold down the left mouse    |"
  echo "|     button and move the mouse to stretch the screen.                        |"
  echo "|                                                                             |"
  echo "+-----------------------------------------------------------------------------+"
  echo
  cOrEAndEnterToContinue

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END expandScreen

# -----------------------------------------------------------------------------
# Make sure the system is the right type
# -----------------------------------------------------------------------------

function checkSystemType {

  subprocessName='checkSystemType'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  str=$(uname -m | cut -c 9,10 )
  if [ "$str" = "$systemModel" ]
  then
    str=`lsattr -El sys0 | grep modelname | grep $modelName`
    if [ "$str" = '' ]
    then
      correctModelNameFound='no'
    else
      correctModelNameFound='yes'
    fi
  else
    correctModelNameFound='no'
  fi

  echo
  if [ "$correctModelNameFound" = 'yes' ]
  then
    echo "INFO: ($scriptName) - Confirmed that system is an RS/6000 $modelName"
    echo
    export processorType='H70'
  else
    writeErr "NSM-95501-E: ERROR($scriptName) - System is not a RS/6000 $modelName.  This MES can"
    writeErr "                                  only be applied to an $modelName."
    writeErr
    eAndEnterToExit
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END checkSystemType

# -----------------------------------------------------------------------------
# Make the log directory
# -----------------------------------------------------------------------------

function createLogDirectory {

  subprocessName='createLogDirectory'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  entryDirectory=`pwd`
  if [ "$uniqueLogDir" = '' ]
  then
    uniqueLogDir="$buildID""_"`date +"%m%d%y_%H%M%S"`
    export uniqueLogDir=$uniqueLogDir  # Unique directory ID for this build
  fi

  str=$(df | grep $logFs)  # logFs (usually /tmp) comes from the override template
  if [ "$str" = '' ]  # If the fs is not present in the df
  then
    cd $logFs >/dev/null 2>&1
    rc=$?
    if [ rc -gt 0 ]
    then
      echo "INFO: ($scriptName) - Filesystem $logFs was not present.  Will attempt to"
      echo "                      create a directory with same name off of root."
      echo
      mkdir $logFs >/dev/null 2>&1
      rc=$?
      if [ rc -gt 0 ]
      then
        writeErr "NSM-95502-E: ERROR($scriptName) - Could not create directory $logFs.  Please"
        writeErr "                                  mount the filesystem or create a directory"
        writeErr "                                  with this name."
        writeErr
        eAndEnterToExit
      else
        echo "INFO: ($scriptName) - Created directory $logFs"
        echo
      fi
    else
      echo "INFO: ($scriptName) - Directory $logFs does exist"
      echo
    fi
  else
    echo "INFO: ($scriptName) - $logFs filesystem is present"
    echo
  fi

  cd $logFs$logDir >/dev/null 2>&1
  rc=$?
  if [ rc -gt 0 ]
  then
    mkdir $logFs$logDir >/dev/null 2>&1
    rc1=$?
    if [ rc1 -gt 0 ]
    then
      writeErr "NSM-95503-E: ERROR($scriptName) - Could not create directory $logFs$logDir"
      writeErr
      eAndEnterToExit
    else
      echo "INFO: ($scriptName) - Created directory $logFs$logDir"
      echo
    fi
  else
    echo "INFO: ($scriptName) - $logFs$logDir directory is present"
    echo
  fi
  cd $entryDirectory     # pwd

  logDir="$logFs$logDir/$uniqueLogDir"
  export logDir="$logDir"     # Directory for logs
  export masterLog="$logDir/$buildID.masterlog.raid5mes"     # Init log name

  mkdir -p $logDir >/dev/null 2>&1
  rc=$?
  if [ rc -gt 0 ]
  then
    writeErr "NSM-95504-E: ERROR($scriptName) - Could not create directory"
    writeErr "                                  $logDir"
    writeErr
    eAndEnterToExit
  else
    echo "INFO: ($scriptName) - Created directory $logDir"
    echo
  fi

  # ---------------------------------------------------------------------------
  # Make sure we can write to log (ie., the fs is not full)

  str=$(echo "Test that we can write to this log" >> $masterLog)
  rc=$?
  if [ "$rc" != '0' ]
  then
    writeErr "NSM-95505-E: ERROR($scriptName) - $masterLog"
    writeErr "                                  cannot be written to.  Please make sure the"
    writeErr "                                  filesystem is not full."
    writeErr
    eAndEnterToExit
  fi

  # ---------------------------------------------------------------------------
  # Erase master log in case it previously exists. Log control parms

  echo "INFO: ($scriptName) - Erasing $masterLog"
  echo "                      in case it previously exists"
  echo
  rm "$masterLog"

  echo "INFO: ($scriptName) > Entering script: displayBuildControl.stsm"
  echo

  theCommand="displayBuildControl.stsm < $buildControlFile >> $masterLog"
  echo "INFO: ($scriptName) - Issuing command: $theCommand"
  echo
  eval $theCommand
  theCommandRc=$?

  if [ "$theCommandRc" != '0' ]
  then
    writeErr "NSM-95506-E: ERROR($scriptName) - There was a problem writing to"
    writeErr "                                  $masterLog"
    writeErr
    eAndEnterToExit
  fi

  echo "INFO: ($scriptName) < Exiting script: displayBuildControl.stsm"
  echo

  # ---------------------------------------------------------------------------
  # Export DSM_LOG to direct error message to the log directory

  export DSM_LOG=$logDir

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END createLogDirectory

# -----------------------------------------------------------------------------
# Verify the operating system is at the minimum required level
# -----------------------------------------------------------------------------

function checkOperatingSystemLevel {

  subprocessName='checkOperatingSystemLevel'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  theOslevelString=`oslevel`

  minRequiredLevel1=$minimumOSVersion
  minRequiredLevel2=$minimumOSRelease
  minRequiredLevel3=$minimumOSLevel

  oslevel1=`echo $theOslevelString | cut -f1 -d'.'`
  oslevel2=`echo $theOslevelString | cut -f2 -d'.'`
  oslevel3=`echo $theOslevelString | cut -f3 -d'.'`

  oslevelCheckPassed='no'
  if [ $oslevel1 -gt $minRequiredLevel1 ]
  then
    oslevelCheckPassed='yes'     # OS at greater than required level
  elif [ $oslevel1 -lt $minRequiredLevel1 ]
  then
    : # Fall through, oslevel less than required
  else
    if [ $oslevel2 -gt $minRequiredLevel2 ]     # OS at greater than required level
    then
      oslevelCheckPassed='yes'
    elif [ $oslevel2 -lt $minRequiredLevel2 ]
    then
      : # Fall through, oslevel less than required
    else
      if [ $oslevel3 -gt $minRequiredLevel3 ]     # OS at greater than required level
      then
        oslevelCheckPassed='yes'
      elif [ $oslevel3 -lt $minRequiredLevel3 ]
      then
        : # Fall through, oslevel less than required
      else
        oslevelCheckPassed='yes' # os at min required level
      fi
    fi
  fi

  echo "INFO: ($scriptName) - The operating system level is at: $theOslevelString"
  if [ "$oslevelCheckPassed" = 'no' ]
  then
    writeErr
    writeErr "NSM-95507-E: ERROR($scriptName) - The operating system level is not at the"
    writeErr "                                  minimum required level of $minRequiredLevel1.$minRequiredLevel2.$minRequiredLevel3"
    writeErr
    eAndEnterToExit
  else
    echo "                      This is >= to the minimum required level of $minRequiredLevel1.$minRequiredLevel2.$minRequiredLevel3"
    echo
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END checkOperatingSystemLevel

# -----------------------------------------------------------------------------
# This function checks in the $raid5mesInfoFile to see if the point of no
# return was reached, and removes the file accordingly if it has not been
# reached.  Point of no return is after dynamicGetADSMresource has been
# executed successfully.
# -----------------------------------------------------------------------------

function pointOfNoReturn {

  subprocessName='pointOfNoReturn'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # Display error messages to STDERR

  writeErr ()
  {
    print "$*" >&2
  }

  wasPointOfNoReturnReached=`cat $raid5mesInfoFile 2> /dev/null | grep pointOfNoReturn | awk '{print $2}'`
  if [ "$wasPointOfNoReturnReached" = '' ]
  then
    rmFileCommand="rm $raid5mesInfoFile"
    echo "INFO: ($scriptName) - Issuing command: $rmFileCommand"
    eval $rmFileCommand 2> /dev/null
    echo 
    exitingSubprocess $subprocessName $subprocessTypeFunction
    programRC=1
    exit $programRC
  else
    echo "NSM-95575-W: WARNING($scriptName) - The $raid5mesInfoFile will"
    echo "                                    NOT be deleted!  Point of no return has"
    echo "                                    been reached when dynamicGetADSMresource"
    echo "                                    was previously called."
    echo
    eAndEnterToExit
  fi

}  # END pointOfNoReturn

# -----------------------------------------------------------------------------
# This is the non-exiting version of the function above
# -----------------------------------------------------------------------------

function pointOfNoReturnWithNoExit {

  subprocessName='pointOfNoReturnWithNoExit'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # Display error messages to STDERR

  writeErr ()
  {
    print "$*" >&2
  }

  wasPointOfNoReturnReached=`cat $raid5mesInfoFile 2> /dev/null | grep pointOfNoReturn | awk '{print $2}'`
  if [ "$wasPointOfNoReturnReached" = '' ]
  then
    rmFileCommand="rm $raid5mesInfoFile"
    echo "INFO: ($scriptName) - Issuing command: $rmFileCommand"
    echo
    eval $rmFileCommand 2> /dev/null
  else
    echo "NSM-95575-W: WARNING($scriptName) - The $raid5mesInfoFile will"
    echo "                                    NOT be deleted!  Point of no return has"
    echo "                                    been reached when dynamicGetADSMresource"
    echo "                                    was previously called."
    echo
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END pointOfNoReturnWithNoExit

# -----------------------------------------------------------------------------
# Initialize hdisks that exist prior to this MES
# -----------------------------------------------------------------------------

function initHdiskListPriorToMes {

  subprocessName='initHdiskListPriorToMes'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  if [ -f $raid5mesInfoFile ]
  then
    clear
    echo
    echo "The RAID 5 MES progress log \"$raid5mesInfoFile\" already exists."
    arraysExistingAtStart=''
    existingHdiskList=''

    while read inputLine
    do
      inputKeyword=`echo $inputLine | awk '{print $1}'`
      case $inputKeyword in 
        ('existingHdisk')
          inputHdisk=`echo $inputLine | awk '{print $2}'`
          existingHdiskList="$existingHdiskList $inputHdisk"
          : ;;
        (*)
          : ;;
      esac
    done < $raid5mesInfoFile

    function displayHeader {
      clear
      echo                 
      echo "This indicates the RAID 5 MES was previously started and never completed."
      echo "Verify the following hdisks (arrays) existed prior to the RAID 5 MES:"
      echo
      displayExistingHdiskList=`echo $existingHdiskList | cut -b1-`
      echo "\"$displayExistingHdiskList\""
      echo 
      echo 
    }  # END displayHeader

    function displayMenu {
      echo "[1] Continue, pre-existing hdisks are correct."
      echo
      echo "[2] Exit, pre-existing hdisks are NOT correct.  Delete"
      echo "    $raid5mesInfoFile so the MES can be re-started."
      echo
      echo "[3] Exit, pre-existing hdisks are NOT correct.  DO NOT delete"
      echo "    $raid5mesInfoFile."
      echo
    }  # END displayMenu

    displayHeader
    response=0
    until [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
    do
      displayMenu
      read response?"Please make your selection [1,2,3]: "
      if [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
      then
        : # Pass through
      else
        displayHeader
        echo "-> $response <- is an invalid response!  Please try again."
      fi
      echo
      echo
    done

    case $response in
      (1)
        echo "INFO: ($scriptName) - User selected Option [1]:"
        echo "                      Continue, pre-existing hdisks are correct."
        echo
        ;;
      (2)
        echo "INFO: ($scriptName) - User selected Option [2]:"
        echo "                      Exit, pre-existing hdisks are NOT correct.  Delete"
        echo "                      $raid5mesInfoFile so the MES can be"
        echo "                      re-started."
        echo
        pointOfNoReturn
        ;;
      (3)
        echo "INFO: ($scriptName) - User selected Option [3]:"
        echo "                      Exit, pre-existing hdisks are NOT correct.  DO NOT"
        echo "                      delete $raid5mesInfoFile."
        echo 
        eAndEnterToExit
        ;;
    esac

  else
    echo "startRaid5mes `date +'%m %d %Y'" > $raid5mesInfoFile
    existingHdiskList=`lsvg -p adsmvg | grep hdisk | awk '{print $1}'`
    for curHdisk in $existingHdiskList
    do
      echo "existingHdisk $curHdisk containing `ssaxlate -l $curHdisk`" >> $raid5mesInfoFile
    done

    function displayHeader {
      clear
      echo
      echo "The following arrays exist prior to this MES:"
      echo
      displayExistingHdiskList=`echo $existingHdiskList | cut -b1-`
      echo "\"$displayExistingHdiskList\""
      echo
      echo "Record these hdisks.  If the MES must be restarted you will be required to"
      echo "confirm these hdisks are the arrays that existed prior to the MES."
      echo
      echo
    }  # END displayHeader

    function displayMenu {
      echo "[1] Continue, I have recorded the hdisks that existed prior to the RAID 5 MES."
      echo
      echo "[2] Exit."
      echo
    }  # END displayMenu

    displayHeader
    response=0
    until [ "$response" = '1' ] || [ "$response" = '2' ]
    do
      displayMenu
      read response?"Please make your selection [1,2]: "
      if [ "$response" = '1' ] || [ "$response" = '2' ]
      then
        : # Pass through
      else
        displayHeader
        echo "-> $response <- is an invalid response!  Please try again."
      fi
      echo
      echo
    done

    case $response in
      (1)
        echo "INFO: ($scriptName) - User selected Option [1]:"
        echo "                      Continue, pre-existing hdisks are recorded."
        echo
        ;;
      (2)
        echo "INFO: ($scriptName) - User selected Option [2] Exit"
        echo
        pointOfNoReturn
        ;;
    esac
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END initHdiskListPriorToMes

# -----------------------------------------------------------------------------
# Prompt user for input. Verify input is valid.
# -----------------------------------------------------------------------------

function promptUserForInput {

  subprocessName='promptUserForInput'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  writeErr ()
  {
    print "$*" >&2
  }

  detectedDisks=`lsdev -Cc pdisk | grep pdisk | wc -l | awk '{print $1}'`
  numberOfExistingHdisks=`lsvg -p adsmvg | grep hdisk | awk '{print $1}' | wc -l | awk '{print $1}'`
  numberOfEnclosuresDetected=`echo $enclosureIdList_system | wc -w | awk '{print $1}'`

  # ---------------------------------------------------------------------------
  # Ask the first question of how many FC's to add

  notDone="true"
  while [ "$notDone" = "true" ]
  do

    # -------------------------------------------------------------------------
    # Prompt the user for selection of feature code

    function displayHeader {
      clear
      echo
      echo "INFO: ($scriptName) - This is a Model $currentModel with $detectedDisks physical disks"
      echo

      echo "The following feature codes describe the disk allocation for RAID:"
      echo
      echo "FC $featureCode1 - First full drawer (default base model)"
      echo "FC $featureCode2 - Second full drawer"
      echo "FC $featureCode3 - Third full drawer"
      echo
      echo "+--------+-------------------+-----------+-------+"
      echo "|        |                   |           | Num   |"
      echo "|        |                   |   Disks   | Disks |"
      echo "| Option |   Feature Code    | From - To | Added |"
      echo "+--------+-------------------+-----------+-------+"
      echo "|   1    |      FC $featureCode2      | 16 to 32  |  16   |"
      echo "|   2    | FC $featureCode2 / FC $featureCode3 | 16 to 48  |  32   |"
      echo "|   3    |      FC $featureCode3      | 32 to 48  |  16   |"
      echo "+--------+-------------------+-----------+-------+"
      echo
      echo
    }  # END displayHeader

    function displayMenu {
      echo "[1] Have 1 drawer, adding 1 more."
      echo
      echo "[2] Have 1 drawer, adding 2 more."
      echo
      echo "[3] Have 2 drawers, adding 1 more."
      echo
      echo "[4] Exit."
      echo
    }  # END displayMenu

    displayHeader
    response=0
    until [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ] || [ "$response" = '4' ]
    do
      displayMenu
      read response?"Please make your selection [1,2,3,4]: "
      if [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ] || [ "$response" = '4' ]
      then
        : # Pass through
      else
        displayHeader
        echo "-> $response <- is an invalid response!  Please try again."
      fi
      echo
      echo
    done

    failedDrawerSelection='false'
    case $response in
      (1)
        function displaySubheader {
          echo "INFO: ($scriptName) - User selected Option [1]:"
          echo "                      Have 1 drawer, adding 1 more."
          echo
        }  # END displaySubheader
        displaySubheader
        disksPriorToMES='16'
        disksAddedWithMES='16'
        enclosuresPriorToMES='1'
        enclosuresAddedWithMES='1'
        (( totalDisks = $disksPriorToMES + $disksAddedWithMES ))
        (( totalEnclosures = $enclosuresPriorToMES + $enclosuresAddedWithMES ))
        if (( $numberOfExistingHdisks == 2 )) && (( $numberOfEnclosuresDetected == 2 )) && (( $detectedDisks == 32 ))
        then
          notDone='false'
          export numDisk=$totalDisks
        else
          failedDrawerSelection='true'
        fi
        ;;
      (2)
        function displaySubheader {
          echo "INFO: ($scriptName) - User selected Option [2]:"
          echo "                      Have 1 drawer, adding 2 more."
          echo
        }  # END displaySubheader
        displaySubheader
        disksPriorToMES='16'
        disksAddedWithMES='32'
        enclosuresPriorToMES='1'
        enclosuresAddedWithMES='2'
        (( totalDisks = $disksPriorToMES + $disksAddedWithMES ))
        (( totalEnclosures = $enclosuresPriorToMES + $enclosuresAddedWithMES ))
        if (( $numberOfExistingHdisks == 2 )) && (( $numberOfEnclosuresDetected == 3 )) && (( $detectedDisks == 48 ))
        then
          notDone='false'
          export numDisk=$totalDisks
        else
          failedDrawerSelection='true'
        fi
        ;;
      (3)
        function displaySubheader {
          echo "INFO: ($scriptName) - User selected Option [3]:"
          echo "                      Have 2 drawers, adding 1 more."
          echo
        }  # END displaySubheader
        displaySubheader
        disksPriorToMES='32'
        disksAddedWithMES='16'
        enclosuresPriorToMES='2'
        enclosuresAddedWithMES='1'
        (( totalDisks = $disksPriorToMES + $disksAddedWithMES ))
        (( totalEnclosures = $enclosuresPriorToMES + $enclosuresAddedWithMES ))
        if (( $numberOfExistingHdisks == 4 )) && (( $numberOfEnclosuresDetected == 3 )) && (( $detectedDisks == 48 ))
        then
          notDone='false'
          export numDisk=$totalDisks
        else
          failedDrawerSelection='true'
        fi
        ;;
      (4)
        echo "INFO: ($scriptName) - User selected Option [4] Exit"
        echo
        pointOfNoReturn
        ;;
    esac

    if [ "$failedDrawerSelection" = 'true' ]
    then
      writeErr "NSM-95508-E: ERROR($scriptName) - That is not a valid option.  Please verify"
      writeErr "                                  your choice and select again."
      writeErr
      cOrEAndEnterToContinue
    fi  # END if [ "$failedDrawerSelection" != 'true' ]
  done  # END while [ "$notDone" = "true" ]

  # ---------------------------------------------------------------------------
  # Ask the second question of how much db space to add

  notDone="true"
  while [ "$notDone" = "true" ]
  do

    # -------------------------------------------------------------------------
    # Prompt user for amount of db space to allocate, or NOT to allocate

    wasPointOfNoReturnReached=`cat $raid5mesInfoFile 2> /dev/null | grep pointOfNoReturn | awk '{print $2}'`
    if [ "$wasPointOfNoReturnReached" = '' ]
    then
      rmFileCommand="rm $raid5mesInfoFile"
      echo "INFO: ($scriptName) - Issuing command: $rmFileCommand"
      echo
      eval $rmFileCommand 2> /dev/null
      function displayHeader {
        clear
        echo
        echo "How much database space would you like to add?"
        echo
        echo "Conditions"
        echo "----------"
        echo "# Specify your answer in GB (gigabytes) - ie., 10 (for 10 GB)"
        echo "# The minimum is $minTsmDbValue (for no additional database space)"
        echo "# The maximum is $maxTsmDbValue"
        echo "# Use integers only (no floating point numbers)"
        echo
        echo
      }  # END displayHeader

      function isValidNumber {
        isNumberGood='yes'
        counter=1
        eachCharacter=$response
        while [ "$eachCharacter" != '' ]
        do
          eachCharacter=`echo $response | cut -c$counter`

          if [ "$eachCharacter" != '' ]
          then
            isValid=`echo $eachCharacter | grep [0-9]`
          fi

          if [ "$isValid" = '' ]
          then
            isNumberGood='no'
          fi

          let counter=counter+1
        done

        if [ "$response" = '' ]
        then
          isNumberGood='no'
        fi

        if [ "$isNumberGood" = 'no' ]
        then
          programRC=1
          return $programRC
        fi
      }  # END isValidNumber

      isCorrectNumberRC=1
      until [ "$isCorrectNumberRC" = '0' ]
      do
        displayHeader

        if [ "$isValidNumberRC" = '1' ]
        then
          echo "-> $response <- is an invalid response!  Please try again."
          echo
          echo
        fi

        read response?"Please enter the desired size [$minTsmDbValue - $maxTsmDbValue]: "
        isValidNumber
        isValidNumberRC=$?

        if [ "$isValidNumberRC" = '0' ]
        then
          if [ "$response" -ge "$minTsmDbValue" ] && [ "$response" -le "$maxTsmDbValue" ]
          then
            export raidDBallocInGb="$response"
            export raidDBallocUnmodified="$response"
            isCorrectNumberRC=0
          else
            isValidNumberRC=1
            isCorrectNumberRC=1
          fi
        fi
      done
    else
      doesPreviousDbSpecExist=`cat $raid5mesInfoFile | grep raidDBalloc`
      previousDBallocSizeInMb=`echo $doesPreviousDbSpecExist | awk '{print $2}'`
      previousDBallocSizeInGb=`echo $doesPreviousDbSpecExist | awk '{print $3}'`
      previousDBallocSizeUnmodified=`echo $doesPreviousDbSpecExist | awk '{print $4}'`
      export raidDBallocInMb="$previousDBallocSizeInMb"
      export raidDBallocInGb="$previousDBallocSizeInGb"
      export raidDBallocUnmodified="$previousDBallocSizeUnmodified"
    fi

    # -------------------------------------------------------------------------
    # Prompt user final time to verify input valid

    function displayHeader {
      clear
      echo

      if [ "$disksPriorToMES" = '16' ] && [ "$disksAddedWithMES" = '32' ]
      then
        if (( $raidDBallocUnmodified == 0 ))
        then
          echo "You have specified no additional database space"
        else
          echo "You have specified $raidDBallocUnmodified GB of additional database space"
        fi
      else
        if (( $raidDBallocUnmodified == 0 ))
        then
          echo "You have specified no additional database space"
        else
          echo "You have specified $raidDBallocInGb GB of additional database space"
        fi
      fi

      if [ "$disksPriorToMES" = '16' ] && [ "$disksAddedWithMES" = '32' ] && [ "$raidDBallocInGb" != '0' ]
      then
        (( raidDBallocInGbDividedByTwo = $raidDBallocInGb / 2 ))
        (( raidDBallocGbPlusOne = $raidDBallocInGbDividedByTwo + 1 ))
        export raidDBallocInGb="$raidDBallocGbPlusOne"
      fi

      echo
      echo
    }  # END displayHeader

    function displayMenu {
      echo "Is this correct?"
      echo
      echo "[1] Yes."
      echo
      echo "[2] No."
      echo
      echo "[3] Exit."
      echo
    }  # END displayMenu

    displayHeader

    if [ "$wasPointOfNoReturnReached" != '' ]
    then
      echo "INFO: ($scriptName) - This database size was specified previously, and cannot"
      echo "                      be changed because Phase 2 has progressed beyond the"
      echo "                      point of no return.  Support must be contacted if a"
      echo "                      change is desired at this time."
      echo
      cOrEAndEnterToContinue
      notDone='false'
    else
      response=0
      until [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
      do
        displayMenu
        read response?"Please make your selection [1,2,3]: "
        if [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
        then
          : # Pass through
        else
          displayHeader
          echo "-> $response <- is an invalid response!  Please try again."
        fi
        echo
        echo
      done

      case $response in
        (1)
          echo "INFO: ($scriptName) - User selected Option [1] Yes"
          echo
          notDone='false'
          ;;
        (2)
          echo "INFO: ($scriptName) - User selected Option [2] No"
          echo
          notDone='true'
          ;;
        (3)
          echo "INFO: ($scriptName) - User selected Option [3] Exit"
          echo
          pointOfNoReturn
          ;;
      esac
    fi  # END if [ "$wasPointOfNoReturnReached" != '' ]
  done  # END while [ "$notDone" = "true" ]

  # -------------------------------------------------------------------------
  # Multiply the previously exported raidDBalloc value by 1024 to convert
  # from GB (gigabytes) to MB (megabytes) for use later by
  # dynamicGetADSMresource

  (( raidDBallocInMb = $raidDBallocInGb * 1024 ))
  if [ "$raidDBallocInGb" = '0' ]
  then
    export raidDBalloc='0'
  else
    export raidDBalloc="$raidDBallocInMb"
  fi

  # -------------------------------------------------------------------------
  # Save out the TSM db size to the RAID 5 MES progress file

  if [ "$doesPreviousDbSpecExist" = '' ]
  then
    if [ "$raidDBallocInGb" = '0' ]
    then
      echo "raidDBalloc 0 0 0" >> $raid5mesInfoFile
    else
      echo "raidDBalloc $raidDBallocInMb $raidDBallocInGb $raidDBallocUnmodified" >> $raid5mesInfoFile
    fi
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END promptUserForInput

# -----------------------------------------------------------------------------
# Changes any disks in system state to free state in the new enclosure(s)
# -----------------------------------------------------------------------------

function cleanUpJbodSystemDisks {

  subprocessName='cleanUpJbodSystemDisks'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  listOfSystemDisks=`ssaraid -Iz -l ssa0 | grep system | awk '{print $1}'`
  if [ "$listOfSystemDisks" != '' ]
  then
    setPdiskUse spare $listOfSystemDisks
    setPdiskUseRC=$?
    if (( $setPdiskUseRC != 0 ))
    then
      writeErr "NSM-95545-E: ERROR($scriptName) - Could not change one or more of the"
      writeErr "                                  following pdisks from 'system' to 'spare':"
      writeErr
      writeErr "                                  $listOfSystemDisks"
      writeErr
      eAndEnterToExit
    fi
  else
    echo "INFO: ($scriptName) - No JBOD disks in 'system' state found"
    echo
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END cleanUpJbodSystemDisks

# -----------------------------------------------------------------------------
# Install updated NSM package if needed
# -----------------------------------------------------------------------------

function installUpdates {

  subprocessName='installUpdates'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # CD to the image directory

  theCommand="cd /cd/usr/sys/inst.images"
  echo "INFO: ($scriptName) - Issuing command: $theCommand"
  echo
  eval $theCommand
 
  # ---------------------------------------------------------------------------
  # Install the filesets in the installation directory
  #
  # Note: The filesets may exist at the same or later level in which case the
  # install will NO-OP

  theCommand=" /usr/lib/instl/sm_inst installp_cmd -a -Q -d '.' -f '_all_latest'  '-c' '-N' '-g' '-X'   '-G'"
  echo "INFO: ($scriptName) - Issuing command: $theCommand"
  echo
  eval $theCommand
 
  # ---------------------------------------------------------------------------
  # Return to the invocation directory

  cd - >/dev/null

  echo
  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END installUpdates

# -----------------------------------------------------------------------------
# Check installed NSM package to see if it's at the minimum level
# -----------------------------------------------------------------------------

function checkInstallUpdates {

  subprocessName='checkInstallUpdates'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # CD to the image directory

  theCommand="cd /cd/usr/sys/inst.images"
  echo "INFO: ($scriptName) - Issuing command: $theCommand"
  echo
  eval $theCommand
 
  # ---------------------------------------------------------------------------
  # Verify the required filesets are installed at the proper level

  subprocessName='checkFilesetLevelsInDir'
  enteringSubprocess $subprocessName $subprocessTypeScript

  $mesBuildscrDir/checkFilesetLevelsInDir

  echo
  exitingSubprocess $subprocessName $subprocessTypeScript

  theRC=$?
  if [ $theRC != 0 ]
  then
    writeErr
    writeErr "NSM-95509-E: ERROR($scriptName) - One or more filesets in the installation"
    writeErr "                                  directory:"
    writeErr
    writeErr "                                  `pwd`"
    writeErr
    writeErr "                                  are not installed at the required level."
    writeErr
    eAndEnterToExit
  fi
 
  # ---------------------------------------------------------------------------
  # Return to the invocation directory

  cd - >/dev/null

  subprocessName='checkInstallUpdates'
  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END checkInstallUpdates

# -----------------------------------------------------------------------------
# Sanity check volume groups
# -----------------------------------------------------------------------------

function sanityCheckVolumeGroups {

  subprocessName='sanityCheckVolumeGroups'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # Set flags

  adsmvgExists='no'
  adsmvg2Exists='no'
  unknownVgExists='no'

  # ---------------------------------------------------------------------------
  # Capture volume group and disk information

  volumeGroupList=`lsvg`
  numberOfSsaPdisks=`lsdev -Cc pdisk | grep SSA | wc -l | awk '{print $1}'`
  
  # ---------------------------------------------------------------------------
  # Recognize volume groups that exist

  offlineVgList=''
  numberOfHdisksInAdsmvg=0
  numberOfHdisksInAdsmvg2=0

  for currentVg in $volumeGroupList
  do
    case $currentVg in
      ('adsmvg')
        adsmvgExists='yes'
        checkIfVgExistsAndActive=$(lsvg -o | grep $currentVg)
        if [ "$checkIfVgExistsAndActive" != '' ]
        then
          numberOfHdisksInAdsmvg=`lsvg -p $currentVg | grep hdisk | wc -l | awk '{print $1}'`
        else
          offlineVgList="$offlineVgList $currentVg"
        fi
        : ;;
      ('adsmvg2')
        adsmvg2Exists='yes'
        checkIfVgExistsAndActive=$(lsvg -o | grep $currentVg)
        if [ "$checkIfVgExistsAndActive" != '' ]
        then
          numberOfHdisksInAdsmvg2=`lsvg -p $currentVg | grep hdisk | wc -l | awk '{print $1}'`
        else
          offlineVgList="$offlineVgList $currentVg"
        fi
        : ;;
      ('rootvg')
        : ;;  #  Ignore
      (*)
        writeErr "NSM-95510-E: ERROR($scriptName) - Unknown volume group '$currentVg' exists."
        writeErr "                                  The only valid volume groups are:"
        writeErr
        writeErr "                                  adsmvg and adsmvg2"
        writeErr
        writeErr "                                  The unknown volume group must be removed"
        writeErr "                                  before continuing."
        writeErr
        eAndEnterToExit
        : ;;
    esac
  done

  if [ "$offlineVgList" != '' ]
  then
    writeErr "NSM-95511-E: ERROR($scriptName) - One or more volume groups are currently"
    writeErr "                                  varied offline.  Corrective action is"
    writeErr "                                  required.  This script cannot proceed until"
    writeErr "                                  all valid volume groups are varied online."
    writeErr "                                  The following volume groups exist but are"
    writeErr "                                  varied offline:"
    writeErr
    for currentOfflineVg in $offlineVgList
    do
      writeErr "                                  $currentOfflineVg"
    done
    writeErr
    eAndEnterToExit
  else
    # -------------------------------------------------------------------------
    # validate volume group and disk counts

    vgValidationError='no'

    # -------------------------------------------------------------------------
    # Audit a 48 disk system 

    if (( $numberOfSsaPdisks == 48 ))
    then
      if [ "$adsmvgExists" = 'yes' ]
      then
        :  # Pass through
      else
        vgValidationError='yes'
      fi

      # -----------------------------------------------------------------------
      # Report vg validation error

      if [ "$vgValidationError" = 'yes' ]
      then
        writeErr "NSM-95512-E: ERROR($scriptName) - SSA validation error for system with"
        writeErr "                                  48 disks.  The \"adsmvg\" volume group"
        writeErr "                                  does not exist.  Corrective action"
        writeErr "                                  is required."
        writeErr
      fi
 
    # -------------------------------------------------------------------------
    # Audit a 32 disk system 

    elif (( $numberOfSsaPdisks == 32 ))
    then
      if [ "$adsmvgExists" = 'yes' ]
      then
        :  # Pass through
      else
        vgValidationError='yes'
      fi
  
      # -----------------------------------------------------------------------
      # Report vg validation error

      if [ "$vgValidationError" = 'yes' ]
      then
        writeErr "NSM-95512-E: ERROR($scriptName) - SSA validation error for system with"
        writeErr "                                  32 disks.  The \"adsmvg\" volume group"
        writeErr "                                  does not exist.  Corrective action"
        writeErr "                                  is required."
        writeErr
      fi

    # -------------------------------------------------------------------------
    # Audit a 16 disk system 

    elif (( $numberOfSsaPdisks == 16 ))
    then
      if [ "$adsmvgExists" = 'yes' ]
      then
        :  # Pass through
      else
        vgValidationError='yes'
      fi

      # -----------------------------------------------------------------------
      # Report vg validation error

      if [ "$vgValidationError" = 'yes' ]
      then
        writeErr "NSM-95512-E: ERROR($scriptName) - SSA validation error for system with"
        writeErr "                                  16 disks.  The \"adsmvg\" volume group"
        writeErr "                                  does not exist.  Corrective action"
        writeErr "                                  is required."
        writeErr
      fi
    else
      writeErr "NSM-95513-E: ERROR($scriptName) - The number of SSA pdisks recognized by the"
      writeErr "                                  system is: '$numberOfSsaPdisks'.  This number is invalid.  The"
      writeErr "                                  system must have exactly 16, 32, or 48 SSA"
      writeErr "                                  pdisks.  Corrective action is required."
      writeErr
      eAndEnterToExit
    fi

    # -------------------------------------------------------------------------
    # Exit from vg validation error

    if [ "$vgValidationError" = 'yes' ]
    then
      eAndEnterToExit
    fi
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END sanityCheckVolumeGroups

# -----------------------------------------------------------------------------
# Verify communications with TSM server
# -----------------------------------------------------------------------------

function verifyCommunicationWithTsmServer {

  subprocessName='verifyCommunicationWithTsmServer'
  enteringSubprocess $subprocessName $subprocessTypeFunction
  
  tsmServerCommunicatedWith=1
  while (( $tsmServerCommunicatedWith != 0 ))
  do
    . vadmin
    tsmServerCommunicatedWith=$?
    echo
    if (( $tsmServerCommunicatedWith == 0 ))
    then
      echo "INFO: ($scriptName) - Communication with the TSM server verified"
      echo
    else
      echo "NSM-95576-W: WARNING($scriptName) - Unable to communicate with the TSM server"
      echo
      echo "  Possible causes:"
      echo
      echo "  -- The server is down.  Instructions state it should be up prior to running"
      echo "     this script."
      echo
      echo "  -- The administrator userid or password is incorrect."
      echo
      echo "  -- The shared memory stanza, ss-m, is missing or invalid.  Check the file:"
      echo "     \"/usr/tivoli/tsm/client/ba/bin/dsm.sys\""
      echo
      echo "  -- The shared memory communication protocol is not enabled.  Try issuing"
      echo "     the \"dsmadmc\" command manually from another window.  Verify the shared"
      echo "     memory communication protocol is enabled in the server options file:"
      echo "     \"/usr/tivoli/tsm/server/bin/dsmserv.opt\""
      echo
      cAndEnterToContinue
      theCommand='ps -eo "%p %c" | grep dsmserv'
      echo "INFO: ($scriptName) - Entering the following command to check if the server"
      echo "                      is up:"
      echo
      echo "                       \"$theCommand\""
      echo
      result=`eval $theCommand | grep -v grep`
      echo "INFO: ($scriptName) - Result: $result"
      echo
      echo "  If the server is up, a result similar to the following message should have"
      echo "  been displayed immediately preceeding this message:"
      echo
      echo "  Example output from ps command: '23724 dsmserv'"
      echo
      echo "  Corrective action required:"
      echo
      echo "  If the server is not up, open a separate window and start the server with"
      echo "  the 'startadsm' command.  If the server is up, other causes for the"
      echo "  communication failure should be investigated"
      echo
      echo "  The communication test with the TSM server will be tried again."
      echo
      cAndEnterToContinue
      echo
    fi
  done

  # ---------------------------------------------------------------------------
  # Set string used for future invocations of dsmadmc

  adminID=$adminUserID  # Exported value from vadmin
  adminPW=$adminPassword  # Exported value from vadmin
  serverID=$serverStanza  # Exported value from vadmin

  if [ "$serverID" = '' ]
  then
    invoc="dsmadmc -id=$adminID -pass=$adminPW"
  else
    invoc="dsmadmc -id=$adminID -pass=$adminPW -server=$serverID"
  fi

  export adsmID=$adminID       # May not need this (HCJ)
  export adsmPSWD=$adminPW     # May not need this
  export adsmServer=$serverID  # May not need this

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END verifyCommunicationWithTsmServer

# -----------------------------------------------------------------------------
# Get list of primary disk storage pools
# -----------------------------------------------------------------------------

function getPrimaryDiskStoragePoolList {

  subprocessName='getPrimaryDiskStoragePoolList'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  theCommand="$invoc -commadelimit select STGPOOL_NAME,ACCESS from STGPOOLS where DEVCLASS=\'DISK\' and POOLTYPE=\'PRIMARY\'"
  theResults=`eval $theCommand | grep -i -E "READ|WRITE|UNAVA"`
  diskpoolList=''
  for currentCombo in $theResults
  do
    diskpoolList="$diskpoolList `echo $currentCombo | cut -f1 -d','`"
  done

  echo "INFO: ($scriptName) - The following primary disk storage pool(s) exist:"
  echo
  echo "                     $diskpoolList"
  echo

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END getPrimaryDiskStoragePoolList

# -----------------------------------------------------------------------------
# Verify TSM stuff
# -----------------------------------------------------------------------------

function verifyTsmStuff {

  subprocessName='verifyTsmStuff'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # Verify all db volumes are sync'd

  theCommand="$invoc -commadelimit select COPY1_NAME,COPY1_STATUS,COPY2_NAME,COPY2_STATUS from DBVOLUMES"
  echo "INFO: ($scriptName) - Issuing command: $theCommand"
  echo
  theResults=`eval $theCommand | grep '/db'`
  echo "INFO: ($scriptName) - TSM DB Mirror status follows:"
  echo
  for currentCombo in $theResults
  do
    copy1_name=`echo $currentCombo | cut -f1 -d','`
    copy1_status=`echo $currentCombo | cut -f2 -d','`
    copy2_name=`echo $currentCombo | cut -f3 -d','`
    copy2_status=`echo $currentCombo | cut -f4 -d','`
    if [ "$copy1_status" = "Sync'd" ] && [ "$copy2_status" = "Sync'd" ]
    then
      echo "  TSM DB Volume set, $copy1_name and $copy2_name are properly Sync'd"
      echo
    else
      echo "NSM-95577-W: WARNING($scriptName) - Problem with mirrored TSM DB set:"
      echo
      echo "    copy 1: dbvolume name: '$copy1_name' dbvolume status: '$copy1_status'"
      echo "    copy 2: dbvolume name: '$copy2_name' dbvolume status: '$copy2_status'"
      echo
      echo "  The script phase being executed is: $userSelectedPhase"
      echo
      echo "  TSM DB volumes that are not sync'd properly during phase 1, indicate"
      echo "  a pre-existing condition that should be corrected prior to continuing."
      echo
      echo "  TSM DB volumes that are not sync'd properly during phase 2, may result"
      echo "  from a previous execution of this script not completing. If this is the"
      echo "  cause, they will be synched once this script completes. As with phase 1,"
      echo "  this may represent a pre-existing condition."
      echo
      echo "  Pre-existing conditions should be corrected BEFORE continuting!"
      echo
      cOrEAndEnterToContinue
    fi
  done

  # ---------------------------------------------------------------------------
  # Verify all log volumes are sync'd

  theCommand="$invoc -commadelimit select COPY1_NAME,COPY1_STATUS,COPY2_NAME,COPY2_STATUS from LOGVOLUMES"
  echo "INFO: ($scriptName) - Issuing command: $theCommand"
  echo
  theResults=`eval $theCommand | grep '/lg'`
  echo "INFO: ($scriptName) - TSM Log Mirror status follows:"
  echo
  for currentCombo in $theResults
  do
    copy1_name=`echo $currentCombo | cut -f1 -d','`
    copy1_status=`echo $currentCombo | cut -f2 -d','`
    copy2_name=`echo $currentCombo | cut -f3 -d','`
    copy2_status=`echo $currentCombo | cut -f4 -d','`
    if [ "$copy1_status" = "Sync'd" ] && [ "$copy2_status" = "Sync'd" ]
    then
      echo "  TSM Log Volume set, $copy1_name and $copy2_name are properly Sync'd"
      echo
    else
      echo "NSM-95578-W: WARNING ($scriptName) - Problem with mirrored TSM Log set:"
      echo
      echo "    copy 1: logvolume name: '$copy1_name' logvolume status: '$copy1_status'"
      echo "    copy 2: logvolume name: '$copy2_name' logvolume status: '$copy2_status'"
      echo
      echo "  TSM Log volumes that are not synched properly result from a pre-existing"
      echo "  condition. This script does not allocate new TSM log volumes."
      echo
      echo "  This condition should be corrected BEFORE continuting!"
      echo
      cOrEAndEnterToContinue
    fi
  done

  # ---------------------------------------------------------------------------
  # Verify at least one disk storage pool exists

  primaryDiskStoragePoolList=$diskpoolList
  numberOfPrimaryDiskStoragePools=`echo $primaryDiskStoragePoolList | wc -w | awk '{print $1}'`

  if (( $numberOfPrimaryDiskStoragePools == 0 ))
  then
    writeErr "NSM-95514-E: ERROR($scriptName) - No primary disk storage pools were found."
    writeErr "                                  At least one primary storage pool must"
    writeErr "                                  exist!"
    writeErr
    eAndEnterToExit
  else
    echo "INFO: ($scriptName) - $numberOfPrimaryDiskStoragePools primary disk storage pool(s) exist:"
    echo
    echo "                     $primaryDiskStoragePoolList"
    echo
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END verifyTsmStuff

# -----------------------------------------------------------------------------
# If there are multiple primary disk storage pools, ask user to select one
# -----------------------------------------------------------------------------

function selectDiskPoolToAddTo {

  subprocessName='selectDiskPoolToAddTo'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # Build list of primary disk storage pools

  numberOfPrimaryDiskStoragePools=`echo $primaryDiskStoragePoolList | wc -w | awk '{print $1}'`
  echo
  if (( $numberOfPrimaryDiskStoragePools == 1 ))
  then
    diskPoolToUse="$primaryDiskStoragePoolList"
  else
    validSpecification='no'
    numberOfEntriesInSelectionList=`echo $primaryDiskStoragePoolList | wc -w | awk '{print $1}'`
    primaryDiskStoragePoolList="$primaryDiskStoragePoolList Abort/Exit"
    while [ "$validSpecification" = 'no' ]
    do
      echo "+-----------------------------------------------------------------------------+"
      echo "|                                                                             |"
      echo "| INFO: ($scriptName) - More than one primary disk storage pools was detected |"
      echo "|                       on this system.  Select one primary disk storage pool |"
      echo "|                       to use.  All of the new disk storage pool volumes     |"
      echo "|                       will be added to the disk storage pool that is        |"
      echo "|                       selected.  Disk storage pool volumes can be moved     |"
      echo "|                       later if desired.                                     |"
      echo "|                                                                             |"
      echo "|  The following example moves disk pool volume /bfaa1dk2 to diskpool2:       |"
      echo "|                                                                             |"
      echo "|  -- query vol, decide what disk volumes to move to the new domain           |"
      echo "|  -- update vol /bfaa1dk2/bf access=readonly, prevent any new data from      |"
      echo "|          being written to the diskpool volume.                              |"
      echo "|  -- move data /bfaa1dk2/bf, moves data to other disk volumes within the     |"
      echo "|          storage pool in which it currently resides                         |"
      echo "|  -- query vol /bfaa1dk2/bf, make sure it is not utilized                    |"
      echo "|  -- delete vol /bfaa1dk2/bf, the volume will not be deleted unless it is    |"
      echo "|          empty                                                              |"
      echo "|  -- define vol diskpool2 /bfaa1dk2/bf, define the volume to the new storage |"
      echo "|          pool                                                               |"
      echo "|                                                                             |"
      echo "+-----------------------------------------------------------------------------+"
      echo 
      sleep 1
      PS3="
Please make your selection: "
      select diskPoolToUse in $primaryDiskStoragePoolList
      do
        if [ "$diskPoolToUse" = '' ]
        then
          echo
          echo "User selected option $REPLY. Invalid selection!"
        elif [ "$REPLY" -le "$numberOfEntriesInSelectionList" ] && [ "$REPLY" -gt "0" ]
        then
          echo
          echo "INFO: ($scriptName) - User selected option $REPLY) $diskPoolToUse"
          echo
          validSpecification='yes'
        else
          echo
          echo "INFO: ($scriptName) - User selected option $REPLY): Abort/Exit"
          echo
          eAndEnterToExit
        fi
  
        if [[ -n $diskPoolToUse ]]
        then
          break
        fi
      done
  
      echo "INFO: ($scriptName) - Confirm that all of the new disk storage pool volumes"
      echo "                      should be added to: $diskPoolToUse"
      echo
      sleep 1  # Seems to be required, or output gets messed up
      PS3="
Please make your selection: "
      select stgpoolSelection in \
          "\"$diskPoolToUse\" is the correct disk storage pool"\
          "\"$diskPoolToUse\" is NOT the correct disk storage pool"
      do
        case $REPLY in
          (1)
            echo
            echo "INFO: ($scriptName) - User selected option 1):"
            echo "                      \"$diskPoolToUse\" is the correct disk storage pool"
            echo
            ;;
          (2)
            echo
            echo "INFO: ($scriptName) - User selected option 2):"
            echo "                      \"$diskPoolToUse\" is NOT the correct disk storage pool"
            echo
            validSpecification='no'
            ;;
          (*)
            echo
            echo "User selected option $REPLY. Invalid selection!"
            ;;
        esac

        if [[ -n $stgpoolSelection ]]
        then
          break
        fi
      done  # select
    done  # while [ "$validSpecification" = 'no' ]
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END selectDiskPoolToAddTo

# -----------------------------------------------------------------------------
# Advise user that no more interaction is needed and give an estimate of time
# -----------------------------------------------------------------------------

function noMoreInputRequired {

  subprocessName='noMoreInputRequired'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  clear
  echo
  echo "+-----------------------------------------------------------------------------+"
  echo "|                                                                             |"
  echo "| INFO: ($scriptName) - This script has collected all of the information      |"
  echo "|                       required to proceed, and should run to completion     |"
  echo "|                       with no further interaction.                          |"
  echo "|                                                                             |"
  echo "|   The remaining steps in this script perform the following functions:       |"
  echo "|                                                                             |"
  echo "|     [1]  Create the AIX logical volume and JFS filesystems for the new      |"
  echo "|          TSM resources                                                      |"
  echo "|                                                                             |"
  echo "|     [2]  Issue 'dsmfmt' to format the new TSM resources                     |"
  echo "|                                                                             |"
  echo "|     [3]  Define the new resources to TSM                                    |"
  echo "|                                                                             |"
  echo "|     [4]  Record information about the changes to 3466                       |"
  echo "|                                                                             |"
  echo "|   The amount of time left to complete the script is dependent upon          |"
  echo "|   several factors:                                                          |"
  echo "|                                                                             |"
  echo "|     [1]  The number of disks being added                                    |"
  echo "|                                                                             |"
  echo "|     [2]  The amount of TSM data base space being added                      |"
  echo "|                                                                             |"
  echo "|     [3]  Other activity that is taking place at the same time the           |"
  echo "|          script is running                                                  |"
  echo "|                                                                             |"
  echo "+-----------------------------------------------------------------------------+"
  echo
  cOrEAndEnterToContinue
  echo "+-----------------------------------------------------------------------------+"
  echo "|                                                                             |"
  echo "|  The following table represents estimates for time required to complete     |"
  echo "|  the RAID 5 MES upgrade with no additional activity on the machine:         |"
  echo "|                                                                             |"
  echo "|                                                                             |"
  echo "|                                   DB Space Added     Estimated Time         |"
  echo "|           From          To           (10 GB)*         (Hrs/Mins)**          |"
  echo "|         --------     --------     --------------     --------------         |"
  echo "|         16 disks     32 disks          No                4H 30M             |"
  echo "|         16 disks     32 disks          Yes               5H                 |"
  echo "|         32 disks     48 disks          No                5H                 |"
  echo "|         32 disks     48 disks          Yes               5H                 |"
  echo "|         16 disks     48 disks          No                9H 30M             |"
  echo "|         16 disks     48 disks          Yes              11H                 |"
  echo "|                                                                             |"
  echo "|                                                                             |"
  echo "|  * NOTE: 10 GB is a sample only.                                            |"
  echo "|  ** Times given have a variability of +/- 10-15 mins                        |"
  echo "|                                                                             |"
  echo "+-----------------------------------------------------------------------------+"
  echo
  cOrEAndEnterToContinue

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END noMoreInputRequired

# -----------------------------------------------------------------------------
# Determine the capacities of the pdisks, ensure all are the same
# -----------------------------------------------------------------------------

function setDiskCapacity {

  subprocessName='setDiskCapacity'
  enteringSubprocess $subprocessName $subprocessTypeFunction 

  capacitiesForPdisksInSystem=`ssaraid -Iz -l ssa0 -t disk | awk '{print $5}'`

  # ---------------------------------------------------------------------------
  # Ensure the capacity of the first pdisk is recognized

  capFirstPdisk=`echo $capacitiesForPdisksInSystem | awk '{print $1}'`
  case "$capFirstPdisk" in
    ('9.1GB')
      export diskType='9GB'
      export hdiskType='raid5m_9'
      : ;;
    ('18.2GB')
      export diskType='18GB'
      export hdiskType='raid5m_18'
      : ;;
    (*)
      writeErr "NSM-95515-E: ERROR($scriptName) - The capacity of the first pdisk is: $capFirstPdisk."
      writeErr "                                  Only pdisk capacities of 9.1GB and 18.2GB"
      writeErr "                                  are supported."
      writeErr
      eAndEnterToExit
      : ;;
  esac

  # ---------------------------------------------------------------------------
  # Ensure all pdisks are the same capacity

  counter=2
  numberOfPdisksInSystem=`echo $capacitiesForPdisksInSystem | wc -w | awk '{print $1}'`

  while [ $counter -le $numberOfPdisksInSystem ]
  do
    capCurrentPdisk=`echo $capacitiesForPdisksInSystem | awk '{print $'$counter'}'`
    if [ "$capCurrentPdisk" != "$capFirstPdisk" ]
    then
      mismatchFound='yes'
    fi
    counter=$(($counter + 1))
  done

  if [ "$mismatchFound" = 'yes' ]
  then
    writeErr "NSM-95516-E: ERROR($scriptName) - All of the pdisks do not have the same"
    writeErr "                                  capacity!  Issue the following command"
    writeErr "                                  to check:"
    writeErr
    writeErr "                                  ssaraid -Iz -l ssa0 -t disk"
    writeErr
    eAndEnterToExit
  else
    echo "INFO: ($scriptName) - Setting environment variable \"hdiskType\" to \"$hdiskType\""
    echo "                      All pdisks have the same capacity of $capFirstPdisk"
    echo
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END setDiskCapacity

# -----------------------------------------------------------------------------
# Validate that the A and B loops are valid
# -----------------------------------------------------------------------------

function validateABLoops {

  subprocessName='validateABLoops'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  if [ "$skipCreateOrderedLoopLists" = 'no' ]
  then
    subprocessName='createOrderedLoopLists'
    enteringSubprocess $subprocessName $subprocessTypeScript

    . createOrderedLoopLists
    createOrderedLoopListsRc=$?

    exitingSubprocess $subprocessName $subprocessTypeScript
  else
    subprocessName='createOrderedLoopLists'
    skippingSubprocess $subprocessName $subprocessTypeScript
    createOrderedLoopListsRc=0
  fi

  if [ "$createOrderedLoopListsRc" != '0' ]
  then
    writeErr "NSM-95517-E: ERROR($scriptName) - There was a problem creating ordered loop"
    writeErr "                                  lists.  Fix the problem and restart the"
    writeErr "                                  script."
    writeErr
    eAndEnterToExit
  elif [ "$createOrderedLoopListsRc" = '0' ]
  then
    echo "INFO: ($scriptName) - A and B loops are valid"
    echo
  fi

  subprocessName='validateABLoops'
  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END validateABLoops

# -----------------------------------------------------------------------------
# Verify the drawers are cabled properly
# -----------------------------------------------------------------------------

function verifyCabling {

  subprocessName='verifyCabling'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  loopIndicator=$1

  # ---------------------------------------------------------------------------
  # Capture the appropriate loop information

  if [ "$loopIndicator" = 'A' ]
  then
    enclosureIdList="$enclosureIdList_ssa0_A1"
    pdiskAllList="$pdiskAllList_ssa0_A1"
    minimumBayNumber=1
    maximumBayNumber=8
  else  # Loop must be B
    enclosureIdList="$enclosureIdList_ssa0_B1"
    pdiskAllList="$pdiskAllList_ssa0_B1"
    minimumBayNumber=9
    maximumBayNumber=16
  fi

  # ---------------------------------------------------------------------------
  # Verify the number of loops and enclosures match

  numberOfEnclosures=`echo $enclosureIdList | wc -w | awk '{print $1}'`
  case $numberOfEnclosures in
    (2)
      if [ $totalDisks = 32 ]
      then
        : # This is correct
      else
        writeErr "NSM-95518-E: ERROR($scriptName) - This system has 2 enclosures with $totalDisks disks."
        writeErr "                                  The system must have exactly 32 disks."
        writeErr
        eAndEnterToExit
      fi
      : ;;
    (3)
      if [ $totalDisks = 48 ]
      then
        : # This is correct
      else
        writeErr "NSM-95518-E: ERROR($scriptName) - This system has 3 enclosures with $totalDisks disks."
        writeErr "                                  The system must have exactly 48 disks."
        writeErr
        eAndEnterToExit
      fi
      : ;;
    (*)
      writeErr "NSM-95519-E: ERROR($scriptName) - The number of enclosure(s): $numberOfEnclosures is not valid."
      writeErr "                                  This system must have 2 or 3 enclosures."
      writeErr
      eAndEnterToExit
  esac

  # ---------------------------------------------------------------------------
  # Traverse the loop and make sure it is cabled properly

  aLoopVerificationErrorOccured='no'
  lastBay=''
  lastEnclosure=''

  for currentEntry in $pdiskAllList
  do
    currentPdisk=`echo $currentEntry | cut -f1 -d':'`
    currentEnclosure=`echo $currentEntry | cut -f9 -d':'`
    currentBay=`echo $currentEntry | cut -f10 -d':'`
    byte1CurrentBay=`echo $currentBay | cut -b1`

    if [ "$byte1CurrentBay" = '0' ]
    then
      currentBay=`echo $currentBay | cut -b2-`
    fi

    if [ "$currentEnclosure" = "$lastEnclosure" ]
    then
      if [ $currentBay -ge $minimumBayNumber ] && [ $currentBay -le $maximumBayNumber ] && [ $currentBay -gt $lastBay ]
      then
        : # Disk in bay is valid
      else
        writeErr "NSM-95520-E: ERROR($scriptName) - Problem with cabling on loop $loopIndicator, currently"
        writeErr "                                  processing: $currentPdisk.  $currentPdisk is plugged"
        writeErr "                                  into bay number $currentBay in enclosure $currentEnclosure.  The"
        writeErr "                                  pdisk must be between bay $minimumBayNumber and bay $maximumBayNumber."
        writeErr "                                  The current bay must also be greater than"
        writeErr "                                  the last bay number: $lastBay."
        writeErr
        writeErr "                                  This is likely due to an SSA cabling"
        writeErr "                                  problem.  This script will exit once all"
        writeErr "                                  audits are complete."
        writeErr
        aLoopVerificationErrorOccured='yes'
      fi
      lastEnclosure=$currentEnclosure
      lastBay=$currentBay
    else
      if [ $currentBay -ge $minimumBayNumber ] && [ $currentBay -le $maximumBayNumber ]
      then
        : # Disk in bay is valid
      else
        writeErr "NSM-95521-E: ERROR($scriptName) - Problem with cabling on loop $loopIndicator, currently"
        writeErr "                                  processing: $currentPdisk.  $currentPdisk is plugged"
        writeErr "                                  into bay number $currentBay in enclosure $currentEnclosure.  The"
        writeErr "                                  pdisk must be between bay $minimumBayNumber and bay $maximumBayNumber."
        writeErr
        writeErr "                                  This is likely due to an SSA cabling"
        writeErr "                                  problem.  This script will exit once all"
        writeErr "                                  audits are complete."
        writeErr
        aLoopVerificationErrorOccured='yes'
      fi
      lastEnclosure=$currentEnclosure
      lastBay=$currentBay
    fi
  done

  if [ "$aLoopVerificationErrorOccured" = 'yes' ]
  then
    eAndEnterToExit
  else
    echo "INFO: ($scriptName) - The $loopIndicator loop is cabled correctly"
    echo
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END verifyCabling

# -----------------------------------------------------------------------------
# Create pdisk lists for spares and arrays to add
# -----------------------------------------------------------------------------

function buildPdiskListsForLoop {

  subprocessName='buildPdiskListsForLoop'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  theLoop=$1
  validationError='no'

  # ---------------------------------------------------------------------------
  # Itemize the arrays and pdisks on the A or B loop

  arraysOnLoop=`ssaraid -Iz -l ssa0 -a network_id=$theLoop | awk '{print $1}'`
  echo "INFO: ($scriptName) - Processing existing arrays on the $theLoop loop:"
  echo
  pdisksInArraysOnLoop=''
  for curArray in $arraysOnLoop
  do
    arrayPdisks=`ssaxlate -l $curArray`
    echo "  Processing pdisk: $curArray"
    pdisksInArraysOnLoop="$pdisksInArraysOnLoop $arrayPdisks"
  done
  echo
  echo "  Array: $curArray"
  echo "  Pdisks in $curArray: $arrayPdisks"
  echo

  # ---------------------------------------------------------------------------
  # Itemize the arrays and pdisks on the A or B loop

  if [ "$theLoop" = 'A' ]
  then
    unassignedPdisksOnLoop=`compareLists unique1 $pdiskList_ssa0_A1 ! $pdisksInArraysOnLoop`
  else  # The loop must be B
    unassignedPdisksOnLoop=`compareLists unique1 $pdiskList_ssa0_B1 ! $pdisksInArraysOnLoop`
  fi

  numberOfUnassignedPdisksOnLoop=`echo $unassignedPdisksOnLoop | wc -w | awk '{print $1}'`
  echo "INFO: ($scriptName) - The following $numberOfUnassignedPdisksOnLoop pdisks exist on the $theLoop loop which are"
  echo "                      not members of any arrays:"
  echo
  echo [*] - $unassignedPdisksOnLoop
  echo

  if [ $numberOfUnassignedPdisksOnLoop -lt $numberOfSparesPerLoop ]
  then
    writeErr "NSM-95522-E: ERROR($scriptName) - The number of pdisks on loop $theLoop is less than"
    writeErr "                                  $numberOfSparesPerLoop which is the minimum number required for"
    writeErr "                                  spares."
    writeErr
    validationError='yes'
  else
    # Total number of unassigned pdisks on the loop
    lastUnassignedPdisk=`echo $unassignedPdisksOnLoop | wc -w | awk '{print $1}'`
    sparePdisksForLoop=''
    spareCounter=0
    while [ $spareCounter -lt $numberOfSparesPerLoop ]
    do
      sparePdisksForLoop="$sparePdisksForLoop `echo $unassignedPdisksOnLoop | awk '{print $'$lastUnassignedPdisk'}'`"
      spareCounter=$(($spareCounter + 1))
      lastUnassignedPdisk=$(($lastUnassignedPdisk - 1))
    done

    unassignedPdisksOnLoop=`compareLists unique1 $unassignedPdisksOnLoop ! $sparePdisksForLoop`
    lastUnassignedPdisk=`echo $unassignedPdisksOnLoop | wc -w | awk '{print $1}'`

    if [ "$disksAddedWithMES" = '16' ] && [ "$lastUnassignedPdisk" = '7' ]
    then
      pdiskList1forArrayToAdd=`echo $unassignedPdisksOnLoop | awk '{print $1, $2, $3, $4, $5, $6, $7}'`
      pdiskList1forArrayToAddNum=`echo $pdiskList1forArrayToAdd | wc -w | awk '{print $1}'`
      if (( $pdiskList1forArrayToAddNum != 7 ))
      then
        writeErr
        writeErr "NSM-95523-E: ERROR($scriptName) - The number of pdisks for the third array to"
        writeErr "                                  add on loop $theLoop is: $pdiskList1forArrayToAddNum"
        writeErr
        writeErr "                                  The number must be exactly 7.  Setting flag"
        writeErr "                                  to exit."
        writeErr
        writeErr "                                  The pdisks in the list are:"
        writeErr "                                  $pdiskList1forArrayToAdd"
        writeErr
        validationError='yes'
      fi
    else
      pdiskList1forArrayToAddNum=0
      pdiskList1forArrayToAdd=''
    fi

    if [ "$disksAddedWithMES" = '32' ] && [ "$lastUnassignedPdisk" = '14' ]
    then
      pdiskList2forArrayToAdd=`echo $unassignedPdisksOnLoop | awk '{print $8, $9, $10, $11, $12, $13, $14}'`
      pdiskList1forArrayToAdd=`echo $unassignedPdisksOnLoop | awk '{print $1, $2, $3, $4, $5, $6, $7}'`
      pdiskList2forArrayToAddNum=`echo $pdiskList2forArrayToAdd | wc -w | awk '{print $1}'`
      if (( $pdiskList2forArrayToAddNum != 7 ))
      then
        writeErr
        writeErr "NSM-95524-E: ERROR($scriptName) - The number of pdisks for the third array to"
        writeErr "                                  add on loop $theLoop is: $pdiskList2forArrayToAddNum"
        writeErr
        writeErr "                                  The number must be exactly 7.  Setting flag"
        writeErr "                                  to exit."
        writeErr
        writeErr "                                  The pdisks in the list are:"
        writeErr "                                  $pdiskList2forArrayToAdd"
        writeErr
        validationError='yes'
      fi
    else
      pdiskList2forArrayToAddNum=0
      pdiskList2forArrayToAdd=''
    fi

    if [ "$theLoop" = 'A' ]
    then
      sparePdisksForLoopA=$sparePdisksForLoop
      pdiskList1forArrayToAddA=$pdiskList1forArrayToAdd
      pdiskList2forArrayToAddA=$pdiskList2forArrayToAdd
      pdiskList3forArrayToAddA=$pdiskList3forArrayToAdd

      echo "INFO: ($scriptName) - Spare pdisks identified for loop A:"
      echo
      echo "[*] - $sparePdisksForLoopA"
      echo

      if [ "$pdiskList1forArrayToAddA" = '' ]
      then
        writeErr "NSM-95525-E: ERROR($scriptName) - No available pdisks detected!"
        writeErr
        eAndEnterToExit
      fi

      echo "INFO: ($scriptName) - List of pdisks to be added as array(s) to loop A:"
      echo
      echo "[*] - $pdiskList1forArrayToAddA"
      if [ "$pdiskList2forArrayToAddA" != '' ]
      then
        echo "[*] - $pdiskList2forArrayToAddA"
      fi
      if [ "$pdiskList3forArrayToAddA" != '' ]
      then
        echo "[*] - $pdiskList3forArrayToAddA"
      fi
      echo 
    else # the loop must = 'B' 
      sparePdisksForLoopB=$sparePdisksForLoop
      pdiskList1forArrayToAddB=$pdiskList1forArrayToAdd
      pdiskList2forArrayToAddB=$pdiskList2forArrayToAdd
      pdiskList3forArrayToAddB=$pdiskList3forArrayToAdd

      echo "INFO: ($scriptName) - Spare pdisks identified for loop B:"
      echo
      echo "[*] - $sparePdisksForLoopB"
      echo

      if [ "$pdiskList1forArrayToAddB" = '' ]
      then
        writeErr "NSM-95525-E: ERROR($scriptName) - No available pdisks detected!"
        writeErr
        eAndEnterToExit
      fi

      echo "INFO: ($scriptName) - List of pdisks to be added as array(s) to loop B:"
      echo
      echo "[*] - $pdiskList1forArrayToAddB"
      if [ "$pdiskList2forArrayToAddB" != '' ]
      then
        echo "[*] - $pdiskList2forArrayToAddB"
      fi
      if [ "$pdiskList3forArrayToAddB" != '' ]
      then
        echo "[*] - $pdiskList3forArrayToAddB"
      fi
      echo 
    fi
  fi

  if [ "$validationError" = 'yes' ]
  then
    writeErr "NSM-95526-E: ERROR($scriptName) - One or more disk validation errors have"
    writeErr "                                  occured.  Corrective action is required."
    writeErr
    eAndEnterToExit
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END buildPdiskListsForLoop 

# -----------------------------------------------------------------------------
# Create spares and arrays as required
# -----------------------------------------------------------------------------

function createSpares {

  subprocessName='createSpares'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  sparePdisksForLoop=$*

  theCommand="setPdiskUse spare $sparePdisksForLoop"
  echo
  echo "INFO: ($scriptName) - Issuing command: $theCommand"
  echo
  eval $theCommand
  setPdiskUseRc=$?

  if [ "$setPdiskUseRc" != '0' ]
  then
    writeErr
    writeErr "NSM-95527-E: ERROR($scriptName) - Error setting pdisk state to \"spare\" for"
    writeErr
    writeErr "[*] - $sparePdisksForLoop"
    writeErr
    eAndEnterToExit
  fi

  echo
  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END createSpares

# -----------------------------------------------------------------------------
# Create an array as required
# -----------------------------------------------------------------------------

function createAnArray {

  subprocessName='createAnArray'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  thePdiskList=$*

  if [ "$thePdiskList" = '' ]
  then
    writeErr "NSM-95528-E: ERROR($scriptName) - No input specified.  No array created."
    writeErr
    eAndEnterToExit
  else
    theCommand="setPdiskUse free $thePdiskList"
    echo
    echo "INFO: ($scriptName) - Issuing command: $theCommand"
    echo
    eval $theCommand
    setPdiskUseRc=$?
    if [ "$setPdiskUseRc" != '0' ]
    then
      writeErr
      writeErr "NSM-95529-E: ERROR($scriptName) - Error setting pdisk states to \"free\" for:"
      writeErr
      writeErr "[*] - $thePdiskList"
      writeErr
      eAndEnterToExit
    else
      theCommand="createArray6p $thePdiskList"
      echo
      echo "INFO: ($scriptName) - Issuing command: $theCommand"
      echo
      theReturnString=`eval $theCommand`
      if [ "$theReturnString" = 'error' ]
      then
        writeErr
        writeErr "NSM-95530-E: ERROR($scriptName) - Error creating array with pdisks:"
        writeErr
        writeErr "[*] - $thePdiskList"
        writeErr
        eAndEnterToExit
      fi
    fi
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END createAnArray

# -----------------------------------------------------------------------------
# Extend the ADSM volume group as appropriate
# -----------------------------------------------------------------------------

function extendADSMvgRaid {

  echo
  subprocessName='extendADSMvgRaid'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  theVg=$1

  allHdisks=`lsdev -Cc disk | grep SSA | awk '{print $1}'`
  theHdisksToAdd=`compareLists unique1 $allHdisks ! $existingHdiskList`

  # ---------------------------------------------------------------------------
  # Do them one at a time -- otherwise the first error causes rest to fail and
  # we want to identify which ones are bad

  if [ "$theHdisksToAdd" = '' ]
  then
    echo
    echo "INFO: ($scriptName) - All of the hdisks are already in the \"$theVg\" volume"
    echo "                      group"
    echo
  else
    echo "INFO: ($scriptName) - Adding the following hdisks to the \"$theVg\""
    echo "                      volume group:"
    echo
    echo "[*] - $theHdisksToAdd"
    echo
    hdisksInVg=`lsvg -p $theVg | grep -v PV_NAME | grep -v $theVg | awk '{print $1}'`
    for currentHdisk in $theHdisksToAdd
    do 
      currentHdisk="$currentHdisk "
      checkIfHdiskAlreadyInVg=`echo $hdisksInVg | grep $currentHdisk`
      echo "INFO: ($scriptName) - Current hdisk is: $currentHdisk"
      echo
      if [ "$checkIfHdiskAlreadyInVg" = '' ]
      then
        echo "INFO: ($scriptName) - $currentHdisk does not already exist in \"$theVg\""
        echo
        extendCommand="extendvg -f '$theVg' '$currentHdisk'"
        echo "INFO: ($scriptName) - Issuing command: $extendCommand"
        echo
        eval $extendCommand 2> /dev/null
        extendCommandRC=$?
        if [ "$extendCommandRC" != '0' ]
        then
          writeErr "NSM-95531-E: ERROR($scriptName) - Extend volume group command failed on"
          writeErr "                                  \"$currentHdisk\""
          writeErr
          eAndEnterToExit
        fi
      else
        echo "INFO: ($scriptName) - $currentHdisk is already in \"$theVg\"!"
        echo
      fi
    done
  fi

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END extendADSMvgRaid

# -----------------------------------------------------------------------------
# Create the resource lists
# -----------------------------------------------------------------------------

function createTheTSMresourceLists {

  subprocessName='createTheTSMresourceLists'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # Create the array hdisk lists for loop A

  #  allArraysOnLoopA=`ssaraid -Iz -l ssa0 -a network_id=A | awk '{print $1}'`
  allArraysOnLoopA=`ssaraid -Iz -l ssa0 -a network_id=A | grep hdisk | awk '{print $1}'`
  newArraysOnLoopA=`compareLists unique1 $allArraysOnLoopA ! $existingHdiskList`
  newArrayHdiskLoopAnum=`echo $newArraysOnLoopA | wc -w | awk '{print $1}'`
  existingHdiskLoopA=`compareLists match $allArraysOnLoopA ! $existingHdiskList`

  export arrayHdiskLoopA="$existingHdiskLoopA $newArraysOnLoopA"

  echo "INFO: ($scriptName) - Hdisk array lists for loop A:"
  echo
  echo "All arrays on loop A"
  echo "--------------------"
  echo $allArraysOnLoopA
  echo
  echo "New arrays on loop A"
  echo "--------------------"
  echo $newArraysOnLoopA
  echo
  echo "Number of new hdisk arrays on loop A"
  echo "------------------------------------"
  echo $newArrayHdiskLoopAnum
  echo
  echo "Existing hdisks on loop A"
  echo "-------------------------"
  echo $existingHdiskLoopA
  echo

  # ---------------------------------------------------------------------------
  # Create the array hdisk lists for loop B

  #  allArraysOnLoopB=`ssaraid -Iz -l ssa0 -a network_id=B | awk '{print $1}'`
  allArraysOnLoopB=`ssaraid -Iz -l ssa0 -a network_id=B | grep hdisk | awk '{print $1}'`
  newArraysOnLoopB=`compareLists unique1 $allArraysOnLoopB ! $existingHdiskList`
  newArrayHdiskLoopBnum=`echo $newArraysOnLoopB | wc -w | awk '{print $1}'`
  existingHdiskLoopB=`compareLists match $allArraysOnLoopB ! $existingHdiskList`

  export arrayHdiskLoopB="$existingHdiskLoopB $newArraysOnLoopB"

  echo "INFO: ($scriptName) - Hdisk array lists for loop B:"
  echo
  echo "All arrays on loop B"
  echo "--------------------"
  echo $allArraysOnLoopB
  echo
  echo "New arrays on loop B"
  echo "--------------------"
  echo $newArraysOnLoopB
  echo
  echo "Number of new hdisk arrays on loop B"
  echo "------------------------------------"
  echo $newArrayHdiskLoopBnum
  echo
  echo "Existing hdisks on loop B"
  echo "-------------------------"
  echo $existingHdiskLoopB
  echo

  readyToContinue='yes'
  if [ "$newArrayHdiskLoopAnum" = '0' ]
  then
    writeErr "NSM-95532-E: ERROR($scriptName) - The number of new hdisks on the A loop is 0."
    writeErr "                                  This is not valid."
    writeErr
    readyToContinue='no'
  fi

  if [ $newArrayHdiskLoopBnum = 0 ]
  then
    writeErr "NSM-95532-E: ERROR($scriptName) - The number of new hdisks on the B loop is 0."
    writeErr "                                  This is not valid."
    writeErr
    readyToContinue='no'
  fi

  if [ "$newArrayHdiskLoopAnum" != "$newArrayHdiskLoopBnum" ]
  then
    writeErr "NSM-95533-E: ERROR($scriptName) - The number of new hdisks on loop A is: $newArrayHdiskLoopAnum"
    writeErr "                                  The number of new hdisks on loop B is: $newArrayHdiskLoopBnum"
    writeErr "                                  They must be equal!"
    writeErr
    readyToContinue='no'
  fi

  if [ "$readyToContinue" = 'no' ]
  then
    eAndEnterToExit
  fi

  export firstDiskSet=$((`echo $existingHdiskLoopA | wc -w | awk '{print $1}'` + 1))
  echo "INFO: ($scriptName) - The first disk to add is position $firstDiskSet on both the A and B"
  echo "                      hdisk lists"
  echo

  # Do not need this, but will leave it in and commented out for now
  #  lgList1=`dynamicGetADSMresource lg all $firstDiskSet 3 both`

  echo "INFO: ($scriptName) > Entering script: dynamicGetADSMresource"
  echo
  dbList1=`dynamicGetADSMresource db all $firstDiskSet 3 both`
  dbList1Rc=$?
  if [ "$dbList1Rc" != '0' ]
  then
    writeErr
    writeErr "NSM-95534-E: ERROR($scriptName) - Failure in script dynamicGetADSMresource!"
    writeErr "                                  Fix problem and re-run script."
    writeErr
    eAndEnterToExit
  fi
  echo 
  echo "INFO: ($scriptName) < Exiting script: dynamicGetADSMresource"
  echo

  echo "INFO: ($scriptName) > Entering script: dynamicGetADSMresource"
  echo
  bfList1=`dynamicGetADSMresource bf all $firstDiskSet 3 both`
  bfList1Rc=$?
  if [ "$dbList1Rc" != '0' ]
  then
    writeErr
    writeErr "NSM-95534-E: ERROR($scriptName) - Failure in script dynamicGetADSMresource!"
    writeErr "                                  Fix problem and re-run script."
    writeErr
    eAndEnterToExit
  fi
  echo 
  echo "INFO: ($scriptName) < Exiting script: dynamicGetADSMresource"
  echo

  # ---------------------------------------------------------------------------
  # If dynamicGetADSMresource has been called, then the additional db size
  # cannot be changed.  So this flag will be checked in the front of the script
  # to prevent the $raid5mesInfoFile from being deleted, and therefore
  # preserving what was originally entered

  echo "pointOfNoReturn yes" >> $raid5mesInfoFile

  # echo
  # echo "The log list (lgList1:`echo $lgList1 | wc -w | awk '{print $1}'`) is: $lgList1"
  echo
  echo "The database list (dbList1:`echo $dbList1 | wc -w | awk '{print $1}'`) is: $dbList1"
  echo
  echo "The primary bitfile list (bfList1:`echo $bfList1 | wc -w | awk '{print $1}'`) is: $bfList1"
  echo

  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END createTheTSMresourceLists

# -----------------------------------------------------------------------------
# Phase 1a: Verification - Executes everything but vdisk
# -----------------------------------------------------------------------------

function phase1a {

  subprocessName='phase1a'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # Make sure the system is the right type

  subprocessName='checkSystemType'

  if [ "$skipCheckSystemType" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeFunction
    botherMeRoutine
  else
    checkSystemType
  fi

  # ---------------------------------------------------------------------------
  # Determine the current model

  file="/usr/nsm.3466/sysInfo/3466.model"
  if [ -f $file ]
  then
    currentModel=`cat $file | cut -b1-3`
  else
    writeErr "NSM-95535-E: ERROR($scriptName) - Cannot determine model type"
    writeErr
    eAndEnterToExit
  fi

  # ---------------------------------------------------------------------------
  # Create a unique build ID 

  if [ "$1" = '' ]
  then
    buildid=$(date "+%m%d_%H%M%S")  # yymmdd_hhmmss
  else
    buildid="$1"
  fi

  logid=$(hostname)
  firstdrawer=""
  lastdrawer=""
  serverID="ss-m"
  buildControlFile="/cd/template.master"
  tmpDsmadmcOut="/tmp/dsmadmcTempFile"

  # ---------------------------------------------------------------------------
  # Make sure master template exists

  if [[ ! -f $buildControlFile ]]
  then
    writeErr "NSM-95536-E: ERROR($scriptName) - The master build control file does not"
    writeErr "                                  exist!  Please call support."
    writeErr
    eAndEnterToExit
  else
    . $buildControlFile  # Execute the master template
  fi

  # ---------------------------------------------------------------------------
  # Execute override template as appropriate

  if [ "$overrideTemplateFile" = '' ]
  then
    echo "INFO: ($scriptName) - No override template specified"
    echo
  elif [ -f "$overrideTemplateFile" ]
  then
    echo "INFO: ($scriptName) - Override template specified"
    echo
    echo "INFO: ($scriptName) - Issuing command: . $overrideTemplateFile"
    echo
    . $overrideTemplateFile
  else
    writeErr "NSM-95537: ERROR($scriptName) - Override template \"$overrideTemplateFile\""
    writeErr "                                not found"
    writeErr
    eAndEnterToExit
  fi

  # ---------------------------------------------------------------------------
  # Overrides

  export buildType='ec'                # Force EC build to be on
  export buildID="$buildid"
  export uniqueLogDir=$logid           # Override templates!!!

  # ---------------------------------------------------------------------------
  # Make the log directory

  subprocessName='createLogDirectory'

  if [ "$skipCreateLogDirectory" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeFunction
    botherMeRoutine
  else
    createLogDirectory 
  fi

  # ---------------------------------------------------------------------------
  # Verify the operating system is at the minimum required level

  subprocessName='checkOperatingSystemLevel'

  if [ "$skipCheckOperatingSystemLevel" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeFunction
    botherMeRoutine
  else
    checkOperatingSystemLevel
  fi

  # ---------------------------------------------------------------------------
  # Install updated NSM package if needed

  installUpdates

  # ---------------------------------------------------------------------------
  # Check installed NSM package to see if it's at the minimum required level

  checkInstallUpdates

  # ---------------------------------------------------------------------------
  # Sanity check volume groups

  subprocessName='sanityCheckVolumeGroups'

  if [ "$skipSanityCheckVolumeGroups" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeFunction
    botherMeRoutine
  else
    sanityCheckVolumeGroups
  fi

  # ---------------------------------------------------------------------------
  # Verify communications with TSM server

  subprocessName='verifyCommunicationWithTsmServer'
  
  if [ "$skipVerifyCommunicationWithTsmServer" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeScript
    invoc="dsmadmc -id=admin -pass=admin"  # Assume defaults if skipped
    botherMeRoutine
  else
    verifyCommunicationWithTsmServer
  fi

  # ---------------------------------------------------------------------------
  # The writeErr function needs to be defined again because
  # verifyCommunicationWithTsmServer wipes it out for some reason

  writeErr ()
  {
    print "$*" >&2
  }

  # ---------------------------------------------------------------------------
  # Get list of primary disk storage pools

  getPrimaryDiskStoragePoolList

  # ---------------------------------------------------------------------------
  # Verify TSM stuff

  subprocessName='verifyTsmStuff'

  if [ "$skipVerifyTsmStuff" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeFunction
    botherMeRoutine
  else
    verifyTsmStuff
  fi

  # ---------------------------------------------------------------------------
  # Determine the capacities of the pdisks, ensure all are the same

  setDiskCapacity

  # ---------------------------------------------------------------------------
  # Verify the loops are balanced

  subprocessName='verifyBalancedLoops'

  if [ "$skipVerifyBalancedLoops" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeFunction
    botherMeRoutine
  else
    enteringSubprocess $subprocessName $subprocessTypeFunction
    arraysInLoopA=`ssaraid -Iz -l ssa0 -a network_id=A | grep hdisk | awk '{print $1}'`
    arraysInLoopB=`ssaraid -Iz -l ssa0 -a network_id=B | grep hdisk | awk '{print $1}'`
    numberOfArraysInLoopA=`echo $arraysInLoopA | wc -w | awk '{print $1}'`
    numberOfArraysInLoopB=`echo $arraysInLoopB | wc -w | awk '{print $1}'`
    if (( $numberOfArraysInLoopA != numberOfArraysInLoopB ))
    then
      writeErr "NSM-95546-E: ERROR($scriptName) - The number of arrays in loop A ($numberOfArraysInLoopA) do not"
      writeErr "                                  match the number of arrays in loop B ($numberOfArraysInLoopB)!"
      writeErr "                                  Please correct and restart the script."
      writeErr
      eAndEnterToExit
    fi
    exitingSubprocess $subprocessName $subprocessTypeFunction
  fi

  # ---------------------------------------------------------------------------
  # Report Phase 1a status

  phase1acompleted='yes'

  subprocessName='phase1a'
  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END phase1a

# -----------------------------------------------------------------------------
# Phase 1b: Verification - Executes vdisk
# -----------------------------------------------------------------------------

function phase1b {

  subprocessName='phase1b'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # Validate that the A and B loops are valid

  subprocessName='validateABLoops'

  if [ "$skipValidateABLoops" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeFunction
    botherMeRoutine
  else
    validateABLoops
  fi

  # ---------------------------------------------------------------------------
  # Verify disks are in good order and resource names are valid

  subprocessName='vdisk'
  
  if [ "$skipVerifyVdisk" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeScript
    botherMeRoutine
  else
    enteringSubprocess $subprocessName $subprocessTypeScript
    vdisk -c -m -v
    vdiskRC=$?
    if (( $vdiskRC != 0 ))
    then
      eAndEnterToExit
    fi
    exitingSubprocess $subprocessName $subprocessTypeScript
  fi

  # ---------------------------------------------------------------------------
  # Report Phase 1b status

  phase1bcompleted='yes'

  subprocessName='phase1b'
  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END phase1b

# -----------------------------------------------------------------------------
# Phase 2: Configuration
# -----------------------------------------------------------------------------

function phase2 {

  # ---------------------------------------------------------------------------
  # Call Phase 1a again to make sure no problems exist before proceeding with
  # Phase 2

  phase1a

  subprocessName='phase2'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  # ---------------------------------------------------------------------------
  # Changes any disks in system state to free state in the new enclosure(s)

  subprocessName='cleanUpJbodSystemDisks'

  if [ "$skipCleanUpJbodSystemDisks" = 'yes' ]
  then
    skippingSubprocess $subprocessName $subprocessTypeFunction
    botherMeRoutine
  else
    cleanUpJbodSystemDisks
  fi

  # ---------------------------------------------------------------------------
  # Call Phase 1b again to make sure no problems exist before proceeding
  # further with Phase 2

  phase1b 

  # ---------------------------------------------------------------------------
  # Initialize hdisks that exist prior to this MES

  initHdiskListPriorToMes

  # ---------------------------------------------------------------------------
  # Prompt user for input. Verify input is valid.

  promptUserForInput

  # ---------------------------------------------------------------------------
  # If there are multiple primary disk storage pools, ask user to select one

  selectDiskPoolToAddTo

  # ---------------------------------------------------------------------------
  # Advise user that no more interaction is needed and give an estimate of time

  noMoreInputRequired

  # ---------------------------------------------------------------------------
  # Begin time stamp for Phase 2 - UNATTENDED

  echo "+-----------------------------------------------------------------------------+"
  echo "  Phase 2 - UNATTENDED"
  echo
  echo "  Beginning at: `date`"
  echo "+-----------------------------------------------------------------------------+"
  echo

  # ---------------------------------------------------------------------------
  # Verify SSA disks are configured properly

  if [ "$skipVerifySSAInstallation" = 'yes' ]
  then
    echo "INFO: ($scriptName) + Skipping function: verifySSAInstallation{}"
    echo
    botherMeRoutine
  else
    verifyRaid5Installation
    rc=$?
    if [ "$rc" != '0' ]
    then
      writeErr
      writeErr "NSM-95538-E: ERROR($scriptName) - SSA disks are not configured correctly.  Fix"
      writeErr "                                  the problem and restart the script."
      writeErr
      eAndEnterToExit
    fi
  fi

  # ---------------------------------------------------------------------------
  # Verify the drawers are cabled properly

  if [ "$skipVerifyCabling" = 'yes' ]
  then
    echo "INFO: ($scriptName) + Skipping function: verifyCabling{}"
    echo
    botherMeRoutine
  else
    verifyCabling 'A'  # Verify the cabling on the A loop
    verifyCabling 'B'  # Verify the cabling on the B loop
  fi

  # ---------------------------------------------------------------------------
  # Calculate the number of spares per loop

  numberOfSparesPerLoop=$(($totalDisks / 16))

  # ---------------------------------------------------------------------------
  # Create pdisk lists for the A and B loops

  if [ "$skipBuildPdiskListsForLoop" = 'yes' ]
  then
    echo "INFO: ($scriptName) + Skipping function: buildPdiskListsForLoop{}"
    echo
    botherMeRoutine
  else
    buildPdiskListsForLoop 'A'
    buildPdiskListsForLoop 'B'
  fi

  # ---------------------------------------------------------------------------
  # Create spares 

  if [ "$skipCreateSpares" = 'yes' ]
  then
    echo "INFO: ($scriptName) + Skipping function: createSpares{}"
    echo
    botherMeRoutine
  else
    createSpares $sparePdisksForLoopA
    createSpares $sparePdisksForLoopB
  fi

  # ---------------------------------------------------------------------------
  # Create arrays 

  if [ "$skipCreateArrays" = 'yes' ]
  then
    echo "INFO: ($scriptName) + Skipping function: createAnArray{}"
    echo
    botherMeRoutine
  else
    createAnArray $pdiskList1forArrayToAddA

    if [ "$pdiskList2forArrayToAddA" != '' ]
    then
      createAnArray $pdiskList2forArrayToAddA
    fi

    if [ "$pdiskList3forArrayToAddA" != '' ]
    then
      createAnArray $pdiskList3forArrayToAddA
    fi

    createAnArray $pdiskList1forArrayToAddB

    if [ "$pdiskList2forArrayToAddB" != '' ]
    then
      createAnArray $pdiskList2forArrayToAddB
    fi

    if [ "$pdiskList3forArrayToAddB" != '' ]
    then
      createAnArray $pdiskList3forArrayToAddB
    fi
  fi

  # ---------------------------------------------------------------------------
  # If any arrays are rebuilding we need to wait before continuing

  waitForNoRaidsRebuilding

  # ---------------------------------------------------------------------------
  # Extend the ADSM volume group as appropriate

  if [ "$skipExtendADSMvgRaid" = 'yes' ]
  then
    echo
    echo "INFO: ($scriptName) + Skipping function: extendADSMvgRaid{}"
    echo
    botherMeRoutine
  else
    extendADSMvgRaid $tsmvg
  fi

  # ---------------------------------------------------------------------------
  # Create the resource lists

  createTheTSMresourceLists

  # ---------------------------------------------------------------------------
  # Create the journaled filesystems for resources to be added

  if [ "$skipCreateJFS" = 'yes' ]
  then
    echo "INFO: ($scriptName) + Skipping script: createJFS.stsm"
    echo
    botherMeRoutine
  else
    createJFS.stsm "$adsmvg" "$dbList1" "$lgList1" "$bfList1"
    rc=$?
    if [ "$rc" != '0' ]
    then
      writeErr "NSM-95539-E: ERROR($scriptName) - Could not create JFS for TSM resources."
      writeErr "                                  Fix the error and restart the phase."
      writeErr
      eAndEnterToExit
    fi
  fi

  # ---------------------------------------------------------------------------
  # Create the TSM resources

  if [ "$skipCreateADSMresource" = 'yes' ]
  then
    echo "INFO: ($scriptName) + Skipping script: createADSMresource.stsm"
    echo
    botherMeRoutine
  else
    export adsmResourceWaitInterval='120'
    createADSMresource.stsm "$dbList1" "$lgList1" "$bfList1"
    rc=$?
    if [ "$rc" != '0' ]
    then
      writeErr "NSM-95540-E: ERROR($scriptName) - Could not create TSM resources."
      writeErr
      eAndEnterToExit
    fi
  fi

  # ---------------------------------------------------------------------------
  # Define space to TSM now

  if [ "$skipDefineADSMvol" = 'yes' ]
  then
    echo "INFO: ($scriptName) + Skipping script: defineADSMvol.stsm"
    echo
    botherMeRoutine
  else
    defineADSMvol.stsm db
    rc=$?
    if [ "$rc" != '0' ]
    then
      writeErr "NSM-95541-E: ERROR($scriptName) - Could not define database volumes to TSM."
      writeErr
      eAndEnterToExit
    fi

    defineADSMvol.stsm bf "$diskPoolToUse"
    rc=$?
    if [ "$rc" != '0' ]
    then
      writeErr "NSM-95541-E: ERROR($scriptName) - Could not define disk storage pool volumes"
      writeErr "                                   to TSM."
      writeErr
      eAndEnterToExit
    fi
  fi

  # ---------------------------------------------------------------------------
  # Save out system info

  if [ "$skipSaveSysInfo" = 'yes' ]
  then
    echo "INFO: ($scriptName) + Skipping script: saveSysInfo"
    echo
    botherMeRoutine
  else
    cd /tmp  # This is here so the record 1 and 2 script calls don't fail
    dir="/usr/nsm.3466/sysInfo"
    /usr/nsm.3466/script/saveSysInfo sysInfoaa2.out $adminID $adminPW $serverID
    echo
    echo "Mfg said this system had $diskNum disks." >> $dir/sysInfoaa2.out
    echo "Mfg said this system had $numLib libraries." >> $dir/sysInfoaa2.out
    echo "Mfg said this system had $numDrivePerLib drives in library 1." >> $dir/sysInfoaa2.out
    echo "Mfg said this system had $numDrivePerLib2 drives in library 2." >> $dir/sysInfoaa2.out
    echo

    # -------------------------------------------------------------------------
    # Record disk info needed for replace.ssa

    theCommand="/usr/nsm.3466/script/record.ssa all"
    echo "INFO: ($scriptName) - Issuing command: $theCommand"
    echo
    eval $theCommand
    theCommandRC=$?
    if (( $theCommandRC != 0 ))
    then
      echo
      echo "NSM-95579-W: WARNING($scriptName) - There was a problem recording disk"
      echo "                                    information for replace.ssa.  Issue the"
      echo "                                    following command from an AIX window:"
      echo
      echo "                                    $theCommand"
      echo
      cAndEnterToContinue
    fi

    # -------------------------------------------------------------------------
    # Record disk info needed for record.build.script

    theCommand="/usr/nsm.3466/script/record 1 $adminID $adminPW $serverID"
    echo "INFO: ($scriptName) - Issuing command: $theCommand"
    echo
    eval $theCommand
    theCommandRC=$?
    if (( $theCommandRC != 0 ))
    then
      echo
      echo "NSM-95580-W: WARNING($scriptName) - There was a problem recording disk"
      echo "                                    information.  Issue the following command"
      echo "                                    from an AIX window:"
      echo
      echo "                                    $theCommand"
      echo
      cAndEnterToContinue
    fi

    # -------------------------------------------------------------------------
    # Create record.build.script

    theCommand="/usr/nsm.3466/script/record 2 $adminID $adminPW $serverID"
    echo "INFO: ($scriptName) - Issuing command: $theCommand"
    echo
    eval $theCommand
    theCommandRC=$?
    if (( $theCommandRC != 0 ))
    then
      echo
      echo "NSM-95581-W: WARNING($scriptName) - There was a problem creating the"
      echo "                                    record.build.script file.  Issue the"
      echo "                                    following command from an AIX window:"
      echo
      echo "                                    $theCommand"
      echo
      cAndEnterToContinue
    fi
  fi  

  # ---------------------------------------------------------------------------
  # The writeErr function needs to be defined again because it gets wiped out
  # for some reason

  writeErr ()
  {
    print "$*" >&2
  }

  # ---------------------------------------------------------------------------
  # Put addauto back into the initialization table

  theCommand="lsitab autosrvr"
  theResult=`eval $theCommand`
  theResultRC=$?
  if (( $theResultRC == 0 )) && [ "$theCommand" != '' ]
  then
    : # Fall through
  else
    echo
    echo "INFO: ($scriptName) - Adding 'autosrvr' into the inittab"
    echo
    theCommand="mkitab 'autosrvr:2:once:/usr/nsm.3466/script/startadsm all'"
    echo "INFO: ($scriptName) - Issuing command: $theCommand"
    echo
    theResult=`eval $theCommand 2>&1`
    theResultRC=$?

    if (( $theResultRC != 0 ))
    then
      writeErr "NSM-95582-W: WARNING($scriptName) - There was a problem adding 'autosrvr' into"
      writeErr "                                    the inittab.  The mkitab command produced"
      writeErr "                                    the following error:"
      writeErr
      writeErr "                                    $theResult"
      writeErr
    fi
  fi

  # ---------------------------------------------------------------------------
  # Delete the RAID 5 MES information file

  echo "INFO: ($scriptName) - Deleting the RAID 5 MES progress log"
  echo
  rm $raid5mesInfoFile

  # ---------------------------------------------------------------------------
  # Customization complete

  echo "INFO: ($scriptName) - PHASE 2 HAS COMPLETED SUCCESSFULLY"
  echo
  phase2completed='yes'

  # ---------------------------------------------------------------------------
  # End time stamp for Phase 2 - UNATTENDED

  echo "+-----------------------------------------------------------------------------+"
  echo "  Phase 2 - UNATTENDED"
  echo
  echo "  Ended at: `date`"
  echo "+-----------------------------------------------------------------------------+"
  echo

  subprocessName='phase2'
  exitingSubprocess $subprocessName $subprocessTypeFunction

  programRC=0
  return $programRC

}  # END phase2

# -----------------------------------------------------------------------------
# Select a phase for processing
# -----------------------------------------------------------------------------

function pickAphase {

  subprocessName='pickAphase'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  function displayHeader {
    clear
    echo
    if [ "$phase1acompleted" = 'yes' ] && [ "$phase1bcompleted" = 'yes' ]
    then
      echo "INFO: ($scriptName) - PHASE 1 HAS COMPLETED SUCCESSFULLY"
      echo
    fi
    echo "+-----------------------------------------------------------------------------+"
    echo "|                                                                             |"
    echo "| Make a selection to run the required phase:                                 |"
    echo "|                                                                             |"
    echo "|   [1]  Phase 1:                                                             |"
    echo "|                                                                             |"
    echo "|     -- Install the minimum NSM package level required for this MES          |"
    echo "|                                                                             |"
    echo "|        NOTE: If this level or a later level is already installed, the       |"
    echo "|              install process will simply NO-OP the install                  |"
    echo "|                                                                             |"
    echo "|     -- Executes verification checks against the current disks.              |"
    echo "|                                                                             |"
    echo "|   Phase 1 is run prior to modifying the 3466 system.  Problems              |"
    echo "|   encountered MUST be corrected prior to continuing.                        |"
    echo "|                                                                             |"
    echo "|   [2]  Phase 2:                                                             |"
    echo "|                                                                             |"
    echo "|     -- Configures the newly added disks to the system.                      |"
    echo "|                                                                             |"
    echo "|   [3]  Exit                                                                 |"
    echo "|                                                                             |"
    echo "| NOTE: Both phase 1 and phase 2 may be re-started in the event the script    |"
    echo "|       does not complete in a previous instance.                             |"
    echo "|                                                                             |"
    echo "+-----------------------------------------------------------------------------+"
    echo
    echo
  }  # END displayHeader

  function displayMenu {
    echo "[1] Phase 1"
    echo
    echo "[2] Phase 2"
    echo
    echo "[3] Exit"
    echo
  }  # END displayMenu

  displayHeader
  response=0
  until [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
  do
    displayMenu
    read response?"Please make your selection [1,2,3]: "
    if [ "$response" = '1' ] || [ "$response" = '2' ] || [ "$response" = '3' ]
    then
      : # Pass through
    else
      displayHeader
      echo "-> $response <- is an invalid response!  Please try again."
    fi
    echo
    echo
  done

  case $response in
    (1)
      echo "INFO: ($scriptName) - User selected Option [1] Phase 1"
      echo
      phase1a
      phase1b
      pickAphase
      ;;
    (2)
      echo "INFO: ($scriptName) - User selected Option [2] Phase 2"
      echo
      phase2
      programRC=0
      return $programRC
      ;;
    (3)
      echo "INFO: ($scriptName) - User selected Option [3] Exit"
      echo
      pointOfNoReturn
      ;;
  esac

  subprocessName='pickAphase'
  exitingSubprocess $subprocessName $subprocessTypeFunction

}  # END pickAphase

# -----------------------------------------------------------------------------
# Mainline
# -----------------------------------------------------------------------------

function mainProcedure {

  enterTimeStamp=`date +"%D %T"`
  echo "INFO: ($scriptName) - ENTERING at $enterTimeStamp"
  echo

  subprocessName='mainProcedure'
  enteringSubprocess $subprocessName $subprocessTypeFunction

  pointOfNoReturnWithNoExit
  expandScreen
  pickAphase

  subprocessName='mainProcedure'
  exitingSubprocess $subprocessName $subprocessTypeFunction

  exitTimeStamp=`date +"%D %T"`
  echo "INFO: ($scriptName) - EXITING at $exitTimeStamp"
  echo

  if [ "$phase2completed" = 'yes' ]
  then
    echo "INFO: ($scriptName) - The 7133 RAID 5 MES upgrade has completed successfully!"
    echo
    cAndEnterToContinue
  fi

}  # END mainProcedure

# -----------------------------------------------------------------------------
# Old text from raid5mes merged into h27367a1.ec
# -----------------------------------------------------------------------------

if [ -d /cd/buildscr ]
then
  export PATH="$PATH:/cd/buildscr"

  # -------------------------------------------------------------------------
  # Ensure current path starts with required directories

  currentPath=`env PATH`
  firstCurrentPathEntry=`echo $currentPath | cut -f1 -d':'`
  secondCurrentPathEntry=`echo $currentPath | cut -f2 -d':'`

  # -------------------------------------------------------------------------
  # Add directories to the beginning of the path as required

  temp3466bin='/tmp/3466bin'
  if [ "$firstCurrentPathEntry" = "$temp3466bin" ]
  then  # OK, this is what we need
    if [ "$secondCurrentPathEntry" = '.' ] || [ "$secondCurrentPathEntry" = './' ]
    then
      echo "INFO: ($scriptName) - PATH already contains $temp3466bin"
      echo
    else
       restOfPath=`echo $currentPath | cut -f2- -d':'`
       echo "INFO: ($scriptName) - Adding $temp3466bin to PATH"
       echo
       export PATH="$temp3466bin:./:$restOfPath"
    fi
  else
    if [ "$firstCurrentPathEntry" = '.' ] || [ "$firstCurrentPathEntry" = './' ]
    then  # OK, this is what we need
      newPath="$temp3466bin:$currentPath"
    else
      newPath="$temp3466bin:./:$currentPath"
    fi
    echo "INFO: ($scriptName) - Adding $temp3466bin to PATH"
    echo
    export PATH="$newPath"
  fi

  buildid=$(date "+%m%d_%H%M")  # mmdd_hhmm
  consoleLogName="$buildid"".console.raid5mes"

  # -------------------------------------------------------------------------
  # Create the console log directory

  echo "INFO: ($scriptName) - Creating console log directory"
  echo
  logDir="/tmp/storsmart/`hostname -s`"
  theCommand="mkdir -p $logDir >/dev/null 2>&1"
  echo "INFO: ($scriptName) - Issuing command: $theCommand"
  echo
  theResult=`eval $theCommand`
  theResultRC=$?
  if (( $theResultRC != 0 ))
  then
    writeErr "NSM-95542-E: ERROR($scriptName) - Could not create the console log directory!"
    writeErr "                                  Corrective action is required."
    writeErr
    eAndEnterToExit
  fi

  # -------------------------------------------------------------------------
  # Setup trap for CTRL-C

  userInterruptString="

  INFO: ($scriptName) - User entered CTRL-C to terminate program

  INFO: ($scriptName) - Script is NOT COMPLETE!
  "
  trap 'echo "$userInterruptString" | tee -a "$logDir/$consoleLogName"' INT

  # -------------------------------------------------------------------------
  # Create the console log archive directory

  echo "INFO: ($scriptName) - Creating console log archive directory"
  echo
  archiveDirectory="/adsmlog/$scriptName"
  theCommand="mkdir -p $archiveDirectory >/dev/null 2>&1"
  echo "INFO: ($scriptName) - Issuing command: $theCommand"
  echo
  theResult=`eval $theCommand`
  theResultRC=$?
  if (( $theResultRC != 0 ))
  then
    writeErr "NSM-95543-E: ERROR($scriptName) - Could not create the console log archive"
    writeErr "                                  directory!  Corrective action is required."
    writeErr
    eAndEnterToExit
  fi

  # -------------------------------------------------------------------------
  # Invoke the mainProcedure to do the bulk of the work

  cd /cd/buildscr
  mainProcedure $buildid 2>&1 | tee $logDir/$consoleLogName

  # -------------------------------------------------------------------------
  # Let user review the console log

  revfile -s -c -a $archiveDirectory $logDir/$consoleLogName

  # -------------------------------------------------------------------------
  # Pass back the return code

  return $programRC

else
  writeErr
  writeErr "NSM-95544-E: ERROR($scriptName) - Directory /cd/buildscr does not exist!"
  writeErr "                                  Corrective action is required."
  writeErr
  eAndEnterToExit
fi
